@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bbs_thread_ui.c,v 1.85 2004/01/17 06:36:31 fuyu Exp $
+ * $Id: bbs_thread_ui.c,v 1.93 2004/01/20 13:49:21 fuyu Exp $
  */
 
 #include "config.h"
@@ -37,6 +37,7 @@
 #include "bbs_thread_view.h"
 #include "boardlist_ui.h"
 #include "bulletin_board_ui.h"
+#include "download_ui.h"
 #include "icon_label.h"
 
 #include "regex_utils.h"
@@ -198,6 +199,7 @@ struct _PopupContext
 
   GtkWidget *popup_window;
   GtkContainer *popup_frame;
+  GtkWidget *image;
 
   PopupContext *parent_context;
 
@@ -436,6 +438,28 @@ preview_thread_view_font(const char *font_name)
 }
 
 
+#if 0
+static void
+popup_window_size_allocate_cb(GtkWidget *widget, GtkAllocation *allocation,
+			      PopupContext *popup_context)
+{
+  fprintf(stderr, "popup_window_size_allocate_cb(%p, %p, %p): w=%d, h=%d\n",
+	  widget, allocation, popup_context,
+	  allocation->width, allocation->height);
+}
+
+
+static void
+popup_window_size_request_cb(GtkWidget *widget, GtkRequisition *requisition,
+			     PopupContext *popup_context)
+{
+  fprintf(stderr, "popup_window_size_request_cb(%p, %p, %p): w=%d, h=%d\n",
+	  widget, requisition, popup_context,
+	  requisition->width, requisition->height);
+}
+#endif
+
+
 static void
 initialize_popup_context(PopupContext *popup_context,
 			 OchushaApplication *application)
@@ -449,6 +473,12 @@ initialize_popup_context(PopupContext *popup_context,
 #endif
 
   popup_context->popup_window = gtk_window_new(GTK_WINDOW_POPUP);
+#if 0
+  g_signal_connect(G_OBJECT(popup_context->popup_window), "size_allocate",
+		   G_CALLBACK(popup_window_size_allocate_cb), popup_context);
+  g_signal_connect(G_OBJECT(popup_context->popup_window), "size_request",
+		   G_CALLBACK(popup_window_size_request_cb), popup_context);
+#endif
   gtk_window_set_resizable(GTK_WINDOW(popup_context->popup_window),
 			   FALSE);
 
@@ -918,7 +948,7 @@ link_mouse_over_cb(BBSThreadView *view, GdkEventMotion *event,
   g_strlcpy(tmp_buffer, link, PATH_MAX);
 
   tmp_link = strstr(tmp_buffer, "<FAKE>");
-  if (application->enable_popup_response)
+  if (application->enable_popup_response || application->enable_popup_image)
     {
       if (tmp_link != NULL)
 	show_popup(application, thread,
@@ -1567,7 +1597,7 @@ insert_bookmark(BBSThreadView *view, OchushaBBSThread *thread, int res_num,
       else
 	info->last_read = bbs_thread_view_create_mark(view);
       if (!application->hide_bookmark)
-	bbs_thread_view_append_widget(view, bookmark);
+	bbs_thread_view_append_widget(view, bookmark, 30, 20);
     }
   else
     {
@@ -1593,6 +1623,36 @@ insert_bookmark(BBSThreadView *view, OchushaBBSThread *thread, int res_num,
 }
 
 
+typedef struct _ThreadRenderingSyncObject
+{
+  pthread_mutex_t lock;
+  pthread_cond_t cond;
+} ThreadRenderingSyncObject;
+
+
+static gboolean
+start_rendering(gpointer data)
+{
+  ThreadRenderingSyncObject *sync_object = (ThreadRenderingSyncObject *)data;
+  if (pthread_mutex_lock(&sync_object->lock) != 0)
+    {
+      fprintf(stderr, "Couldn't lock a mutex.\n");
+      abort();
+    }
+  if (pthread_cond_signal(&sync_object->cond) != 0)
+    {
+      fprintf(stderr, "Couldn't signal a condition.\n");
+      abort();
+    }
+  if (pthread_mutex_unlock(&sync_object->lock) != 0)
+    {
+      fprintf(stderr, "Couldn't unlock a mutex.\n");
+      abort();
+    }
+  return FALSE;
+}
+
+
 static void
 render_bbs_thread(WorkerThread *employee, gpointer args)
 {
@@ -1656,6 +1716,8 @@ render_bbs_thread(WorkerThread *employee, gpointer args)
       						/* regex_t */
       						/* regex_t */
   };
+  ThreadRenderingSyncObject sync_object;
+
 
   /* "\357\274\214\343\200\201\357\274\235"  "" UTF-8ɽ */
   /* "\357\274\220"""UTF-8ɽ */
@@ -1843,6 +1905,52 @@ render_bbs_thread(WorkerThread *employee, gpointer args)
       OCHU_THREADS_LEAVE();
     }
 
+  if (pthread_mutex_init(&sync_object.lock, NULL) != 0)
+    {
+      fprintf(stderr, "Couldn't init a mutex.\n");
+      abort();
+    }
+
+  if (pthread_cond_init(&sync_object.cond, NULL) != 0)
+    {
+      fprintf(stderr, "Couldn't init a condition variable.\n");
+      abort();
+    }
+
+  if (pthread_mutex_lock(&sync_object.lock) != 0)
+    {
+      fprintf(stderr, "Couldn't lock a mutex.\n");
+      abort();
+    }
+
+  /* idleؿϿ */
+  g_idle_add_full(GDK_PRIORITY_REDRAW + 10,
+		  start_rendering, &sync_object, NULL);
+
+  if (pthread_cond_wait(&sync_object.cond, &sync_object.lock) != 0)
+    {
+      fprintf(stderr, "Couldn't wait a condition.\n");
+      abort();
+    }
+
+  if (pthread_mutex_unlock(&sync_object.lock) != 0)
+    {
+      fprintf(stderr, "Couldn't unlock a mutex.\n");
+      abort();
+    }
+
+  if (pthread_mutex_destroy(&sync_object.lock) != 0)
+    {
+      fprintf(stderr, "Couldn't destroy a mutex.\n");
+      abort();
+    }
+
+  if (pthread_cond_destroy(&sync_object.cond) != 0)
+    {
+      fprintf(stderr, "Couldn't destroy a condition.\n");
+      abort();
+    }
+
   ochusha_bbs_thread_parse_responses(thread, buffer,
 				     job_args->start,
 				     job_args->number_of_responses,
@@ -1851,6 +1959,10 @@ render_bbs_thread(WorkerThread *employee, gpointer args)
 				     render_response,
 				     render_broken_response,
 				     end_thread_rendering,
+				     (StartParsingCallback *)OCHU_THREADS_ENTER_FUNC,
+				     (BeforeWaitCallback *)OCHU_THREADS_LEAVE_FUNC,
+				     (AfterWaitCallback *)OCHU_THREADS_ENTER_FUNC,
+				     (EndParsingCallback *)OCHU_THREADS_LEAVE_FUNC,
 				     &context);
 
   update_threadlist_entry(job_args->application, thread);
@@ -1926,8 +2038,6 @@ start_thread_rendering(OchushaBBSThread *real_thread, const gchar *title,
 
   context->title_shown = TRUE;
 
-  OCHU_THREADS_ENTER();
-
   if (context->last_read_number == 0 && info->res_num == 0)
     {
       /* ɤॹξ祿ȥߤˤΤǤ˥ޡդ롣*/
@@ -1951,8 +2061,6 @@ start_thread_rendering(OchushaBBSThread *real_thread, const gchar *title,
       bbs_thread_view_scroll_to_mark(view, info->last_read);
     }
 
-  OCHU_THREADS_LEAVE();
-
   return TRUE;	/* go ahead! */
 }
 
@@ -2018,19 +2126,15 @@ render_response(OchushaBBSThread *real_thread, int number,
     ochusha_bbs_thread_set_number_of_responses_read(thread, number);
 
   info->num_shown = number;
-  update_threadlist_entry(context->application, thread);
+  update_threadlist_entry_real(context->application, thread);
 
   if (a_bone && context->transparent_a_bone)
     {
-      OCHU_THREADS_ENTER();
-
       goto after_response;
     }
 
   snprintf(buffer, 18, "<RES>%d", number);
 
-  OCHU_THREADS_ENTER();
-
   bbs_thread_view_push_tag(view, context->response_header_tag);
   bbs_thread_view_append_text(view, "\n", 1);
   if (context->last_read_number >= 0)
@@ -2219,8 +2323,6 @@ render_response(OchushaBBSThread *real_thread, int number,
 	}
     }
 
-  OCHU_THREADS_LEAVE();
-
   return TRUE;
 }
 
@@ -2267,16 +2369,16 @@ render_broken_response(OchushaBBSThread *real_thread, int number,
     ochusha_bbs_thread_set_number_of_responses_read(thread, number);
 
   info->num_shown = number;
-  update_threadlist_entry(context->application, thread);
+  update_threadlist_entry_real(context->application, thread);
 
   snprintf(buffer, 13, "\n%d", number);
 
-  OCHU_THREADS_ENTER();
-
+  bbs_thread_view_push_tag(view, context->response_header_tag);
   bbs_thread_view_append_text(view, buffer, -1);
   bbs_thread_view_append_text(view, LARGE_COLON, 3);
   bbs_thread_view_append_text(view, LARGE_COLON, 3);
   bbs_thread_view_append_text(view, _("[broken here]\n"), -1);
+  bbs_thread_view_pop_tags(view, context->response_header_tag);
 
   bbs_thread_view_push_tag(view, context->response_paragraph_tag);
   bbs_thread_view_append_text(view, _("[broken here]\n"), -1);
@@ -2301,8 +2403,6 @@ render_broken_response(OchushaBBSThread *real_thread, int number,
 	}
     }
 
-  OCHU_THREADS_LEAVE();
-
   return TRUE;
 }
 
@@ -2353,16 +2453,12 @@ end_thread_rendering(OchushaBBSThread *real_thread, gboolean finished,
 	       _("A Bone occurs in thread %s@%s board"),
 	       thread->title, ochusha_bbs_thread_get_board(thread)->name);
 
-      OCHU_THREADS_ENTER();
-
       if (info->message_id != 0)
 	gtk_statusbar_remove(context->application->statusbar,
 			     thread_netstat_id, info->message_id);
       info->message_id = gtk_statusbar_push(context->application->statusbar,
 					    thread_netstat_id, message);
 
-      OCHU_THREADS_LEAVE();
-
       ochusha_bbs_thread_set_number_of_responses_read(thread, 1);
       /* 0ˤȡɤ߹ߤߤˡå夬ĤäƤΤ̤
        * ˤʤ륿ߥ󥰤Τǡ
@@ -2388,6 +2484,23 @@ process_link(RendererContext *context)
 			     context->link_text_length,
 			     NULL, NULL);
 
+  if (context->application->enable_inline_image)
+    {
+      char *link = context->link;
+      if (g_str_has_suffix(link, ".jpg") || g_str_has_suffix(link, ".png")
+	  || g_str_has_suffix(link, ".gif") || g_str_has_suffix(link, ".JPG")
+	  || g_str_has_suffix(link, ".PNG") || g_str_has_suffix(link, ".GIF"))
+	{
+	  /* ᡼μ¸ */
+	  OchushaApplication *application = context->application;
+	  GtkWidget *image
+	    = ochusha_download_image(application, link,
+				     application->inline_image_width,
+				     application->inline_image_height);
+	  bbs_thread_view_append_widget(context->view, image, 20, 0);
+	}
+    }
+
   if (text != NULL)
     bbs_thread_view_append_text_as_link(context->view, text, -1,
 					context->link);
@@ -2396,6 +2509,7 @@ process_link(RendererContext *context)
 					context->link_text_buffer,
 					context->link_text_length,
 					context->link);
+
   if (text != NULL)
     G_FREE(text);
   G_FREE(context->link);
@@ -2757,109 +2871,6 @@ prepare_buffer_for_popup(OchushaApplication *application,
 }
 
 
-static gboolean need_popup_adjustment = FALSE;
-
-
-static gboolean
-popup_expose_event_cb(GtkWidget *widget, GdkEventExpose *event,
-		      PopupContext *popup_context)
-{
-  GtkRequisition requisition = { 0, 0 };
-
-  MAIN_THREAD_EVENT_CALLBACK();
-
-  gtk_widget_size_request(widget, &requisition);
-
-#if DEBUG_GUI_MOST
-  fprintf(stderr, "popup_expose_event_cb(widget=%p), w=%d, h=%d\n",
-	  widget, requisition.width, requisition.height);
-#endif
-  if (requisition.width > 0 && need_popup_adjustment)
-    {
-      BBS_THREAD_UI_LOCK
-      {
-	need_popup_adjustment = FALSE;
-      }
-      BBS_THREAD_UI_UNLOCK;
-
-      if (popup_context->renderer_context.application->enable_smart_popup_placement)
-	{
-	  GdkScreen *screen = gtk_widget_get_screen(popup_context->popup_window);
-
-	  popup_context->y_pos -= requisition.height;
-	  popup_context->y_pos
-	    = popup_context->y_pos > 0 ? popup_context->y_pos : 0;
-
-	  if (screen != NULL)
-	    {
-	      int screen_width = gdk_screen_get_width(screen);
-	      int widget_width = requisition.width + THREAD_VIEW_EXTRA_WIDTH;
-	      if (screen_width > 0
-		  && (popup_context->x_pos + widget_width) > screen_width)
-		{
-		  popup_context->x_pos = screen_width - widget_width;
-		  if (popup_context->x_pos < 0)
-		    popup_context->x_pos = 0;
-		}
-	    }
-	}
-
-      gtk_widget_set_size_request(widget,
-				  requisition.width
-				  + THREAD_VIEW_EXTRA_WIDTH,
-				  requisition.height);
-      gtk_window_resize(GTK_WINDOW(popup_context->popup_window),
-			requisition.width + THREAD_VIEW_EXTRA_WIDTH,
-			requisition.height);
-
-      if (popup_context->renderer_context.application->enable_smart_popup_placement)
-	gtk_window_move(GTK_WINDOW(popup_context->popup_window),
-			popup_context->x_pos, popup_context->y_pos);
-    }
-
-  ENTER_MAIN_EVENT_LOOP();
-  return FALSE;
-}
-
-
-#if DEBUG_WIDGET_MOST
-/*
- * XXX: Ȥʪˤʤ󥷥ʥġġ
- */
-static gboolean
-popup_map_event_cb(GtkWidget *widget, GdkEvent *event,
-		   PopupContext *popup_context)
-{
-  GtkRequisition requisition = { 0, 0 };
-
-  MAIN_THREAD_EVENT_CALLBACK();
-
-  gtk_widget_size_request(widget, &requisition);
-  fprintf(stderr, "popup_map_event_cb(widget=%p), w=%d, h=%d\n",
-	  widget, requisition.width, requisition.height);
-
-  ENTER_MAIN_EVENT_LOOP();
-  return FALSE;
-}
-
-
-static void
-popup_realize_cb(GtkWidget *widget, PopupContext *popup_context)
-{
-  GtkRequisition requisition = { 0, 0 };
-
-  MAIN_THREAD_EVENT_CALLBACK();
-
-  gtk_widget_size_request(widget, &requisition);
-  fprintf(stderr, "popup_realize_cb(widget=%p), w=%d, h=%d\n",
-	  widget, requisition.width, requisition.height);
-
-  ENTER_MAIN_EVENT_LOOP();
-  return;
-}
-#endif
-
-
 static gboolean
 popup_window_button_press_event_cb(GtkWidget *widget, GdkEventButton *event,
 				   PopupContext *popup_context)
@@ -2886,6 +2897,9 @@ popup_window_enter_notify_event_cb(GtkWidget *widget,
 #if 0	/* ̵̤ʤΤ */
   MAIN_THREAD_EVENT_CALLBACK();
 #endif
+#if 0
+  fprintf(stderr, "enter_notify: %p, %p\n", widget, popup_context);
+#endif
 
   if (popup_context->close_delay_id != 0)
     {
@@ -2920,6 +2934,50 @@ popup_window_leave_notify_event_cb(GtkWidget *widget,
 }
 
 
+static void
+popup_view_size_request_cb(GtkWidget *widget, GtkRequisition *requisition,
+			     PopupContext *popup_context)
+{
+  GdkScreen *screen;
+  int x_pos;
+  int y_pos;
+#if 0
+  fprintf(stderr, "popup_view_size_request_cb(%p, %p, %p): w=%d, h=%d\n",
+	  widget, requisition, popup_context,
+	  requisition->width, requisition->height);
+#endif
+  if (requisition->width <= 0 || requisition->height <= 0)
+    return;
+
+  gtk_widget_set_size_request(widget,
+			      requisition->width
+			      + THREAD_VIEW_EXTRA_WIDTH,
+			      requisition->height);
+  gtk_window_resize(GTK_WINDOW(popup_context->popup_window),
+		    requisition->width + THREAD_VIEW_EXTRA_WIDTH,
+		    requisition->height);
+
+  screen = gtk_widget_get_screen(popup_context->popup_window);
+  x_pos = popup_context->x_pos;
+  y_pos = popup_context->y_pos - requisition->height;
+  y_pos = y_pos > 0 ? y_pos : 0;
+
+  if (screen != NULL)
+    {
+      int screen_width = gdk_screen_get_width(screen);
+      int widget_width = requisition->width + THREAD_VIEW_EXTRA_WIDTH;
+      if (screen_width > 0 && (x_pos + widget_width) > screen_width)
+	{
+	  x_pos = screen_width - widget_width;
+	  if (x_pos < 0)
+	    x_pos = 0;
+	}
+    }
+
+  gtk_window_move(GTK_WINDOW(popup_context->popup_window), x_pos, y_pos);
+}
+
+
 static void
 render_popup_response(PopupContext *popup_context,
 		      OchushaBBSThread *thread, OchushaAsyncBuffer *buffer,
@@ -2961,21 +3019,20 @@ render_popup_response(PopupContext *popup_context,
 		   G_CALLBACK(popup_window_leave_notify_event_cb),
 		   popup_context);
 
-  g_signal_connect_after(G_OBJECT(view), "expose_event",
-			 G_CALLBACK(popup_expose_event_cb), popup_context);
-
-#if DEBUG_WIDGET_MOST
-  g_signal_connect(G_OBJECT(view), "map_event",
-		   G_CALLBACK(popup_map_event_cb), popup_context);
-  g_signal_connect_after(G_OBJECT(view), "realize",
-			 G_CALLBACK(popup_realize_cb), popup_context);
-#endif
+  g_signal_connect(G_OBJECT(view), "size_request",
+		   G_CALLBACK(popup_view_size_request_cb), popup_context);
 
   gtk_widget_show_all(view);
 
   if (popup_context->renderer_context.view != NULL)
     gtk_container_remove(popup_context->popup_frame,
 			 GTK_WIDGET(popup_context->renderer_context.view));
+  else if (popup_context->image != NULL)
+    {
+      gtk_container_remove(popup_context->popup_frame,
+			   popup_context->image);
+      popup_context->image = NULL;
+    }
   gtk_container_add(popup_context->popup_frame, view);
   gtk_widget_show(GTK_WIDGET(popup_context->popup_frame));
 
@@ -3005,6 +3062,10 @@ render_popup_response(PopupContext *popup_context,
 				     TRUE,
 				     NULL, render_response,
 				     render_broken_response, NULL,
+				     (StartParsingCallback *)OCHU_THREADS_ENTER_FUNC,
+				     (BeforeWaitCallback *)OCHU_THREADS_LEAVE_FUNC,
+				     (AfterWaitCallback *)OCHU_THREADS_ENTER_FUNC,
+				     (EndParsingCallback *)OCHU_THREADS_LEAVE_FUNC,
 				     &popup_context->renderer_context);
   OCHU_THREADS_ENTER();
 
@@ -3013,6 +3074,62 @@ render_popup_response(PopupContext *popup_context,
 }
 
 
+#if 0	/* ɬפʤäݤ */
+static void
+image_realize_cb(GtkWidget *widget, PopupContext *unused)
+{
+  GdkWindow *window = gtk_widget_get_parent_window(widget);
+  fprintf(stderr, "realize_cb\n");
+  gdk_window_set_events(window,
+			(gdk_window_get_events(window)
+			 & ~GDK_POINTER_MOTION_HINT_MASK)
+			| GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK);
+}
+#endif
+
+
+static void
+popup_image_size_request_cb(GtkWidget *widget, GtkRequisition *requisition,
+			    PopupContext *popup_context)
+{
+  GdkScreen *screen;
+  int x_pos;
+  int y_pos;
+#if 0
+  fprintf(stderr, "popup_view_size_request_cb(%p, %p, %p): w=%d, h=%d\n",
+	  widget, requisition, popup_context,
+	  requisition->width, requisition->height);
+#endif
+  if (requisition->width <= 0 || requisition->height <= 0)
+    return;
+
+  gtk_widget_set_size_request(widget,
+			      requisition->width,
+			      requisition->height);
+  gtk_window_resize(GTK_WINDOW(popup_context->popup_window),
+		    requisition->width, requisition->height);
+
+  screen = gtk_widget_get_screen(popup_context->popup_window);
+  x_pos = popup_context->x_pos;
+  y_pos = popup_context->y_pos - requisition->height;
+  y_pos = y_pos > 0 ? y_pos : 0;
+
+  if (screen != NULL)
+    {
+      int screen_width = gdk_screen_get_width(screen);
+      int widget_width = requisition->width;
+      if (screen_width > 0 && (x_pos + widget_width) > screen_width)
+	{
+	  x_pos = screen_width - widget_width;
+	  if (x_pos < 0)
+	    x_pos = 0;
+	}
+    }
+
+  gtk_window_move(GTK_WINDOW(popup_context->popup_window), x_pos, y_pos);
+}
+
+
 static void
 show_popup(OchushaApplication *application, OchushaBBSThread *thread,
 	   const gchar *link, int x_pos, int y_pos, BBSThreadView *parent)
@@ -3034,6 +3151,51 @@ show_popup(OchushaApplication *application, OchushaBBSThread *thread,
   fprintf(stderr, "show_popup: link=\"%s\", x_pos=%d, y_pos=%d\n",
 	  link, x_pos, y_pos);
 #endif
+  if (application->enable_popup_image)
+    {
+      if (g_str_has_suffix(link, ".jpg") || g_str_has_suffix(link, ".png")
+	  || g_str_has_suffix(link, ".gif") || g_str_has_suffix(link, ".JPG")
+	  || g_str_has_suffix(link, ".PNG") || g_str_has_suffix(link, ".GIF"))
+	{
+	  /* ᡼μ¸ */
+	  GtkWidget *event_box = gtk_event_box_new();
+	  GtkWidget *image
+	    = ochusha_download_image(application, link,
+				     application->popup_image_width,
+				     application->popup_image_height);
+	  gtk_widget_show(image);
+	  gtk_container_add(GTK_CONTAINER(event_box), image);
+	  gtk_widget_show(event_box);
+#if 0
+	  g_signal_connect(G_OBJECT(event_box), "realize",
+			   G_CALLBACK(image_realize_cb), popup_context);
+#endif
+	  g_signal_connect(G_OBJECT(event_box), "enter_notify_event",
+			   G_CALLBACK(popup_window_enter_notify_event_cb),
+			   popup_context);
+	  g_signal_connect(G_OBJECT(event_box), "leave_notify_event",
+			   G_CALLBACK(popup_window_leave_notify_event_cb),
+			   popup_context);
+	  g_signal_connect(G_OBJECT(image), "size_request",
+			   G_CALLBACK(popup_image_size_request_cb),
+			   popup_context);
+
+	  if (popup_context->renderer_context.view != NULL)
+	    {
+	      gtk_container_remove(popup_context->popup_frame,
+				   GTK_WIDGET(popup_context->renderer_context.view));
+	      popup_context->renderer_context.view = NULL;
+	    }
+	  else if (popup_context->image != NULL)
+	    gtk_container_remove(popup_context->popup_frame,
+				 popup_context->image);
+	  gtk_container_add(popup_context->popup_frame, event_box);
+	  gtk_widget_show(GTK_WIDGET(popup_context->popup_frame));
+	  popup_context->image = event_box;
+	  goto do_popup;
+	}
+    }
+
   if (!prepare_buffer_for_popup(application, thread, link,
 				&response_from, &response_to, &buffer))
     {
@@ -3062,6 +3224,8 @@ show_popup(OchushaApplication *application, OchushaBBSThread *thread,
   render_popup_response(popup_context, thread, buffer,
 			response_from, num_responses);
 
+ do_popup:
+
   popup_context->x_pos = x_pos;
   popup_context->y_pos = y_pos;
 
@@ -3129,72 +3293,9 @@ show_popup_real(PopupContext *popup_context)
   fprintf(stderr, "show_popup_real(): popup_context=%p\n", popup_context);
 #endif
 
-  if (popup_context->renderer_context.view != NULL)
+  if (popup_context->renderer_context.view != NULL
+      || popup_context->image != NULL)
     {
-      int w, h;
-      gboolean use_two_steps_hack = FALSE;
-      GtkRequisition requisition = { 0, 0 };
-      GdkScreen *screen = gtk_widget_get_screen(popup_context->popup_window);
-      int screen_width = screen != NULL ? gdk_screen_get_width(screen) : 0;
-
-      gtk_widget_size_request(GTK_WIDGET(popup_context->renderer_context.view),
-			      &requisition);
-      gtk_widget_get_size_request(GTK_WIDGET(popup_context->renderer_context.view),
-				  &w, &h);
-
-#if DEBUG_POPUP
-      fprintf(stderr, "widget=%p, w=%d, h=%d\n",
-	      popup_context->renderer_context.view,
-	      requisition.width, requisition.height);
-#endif
-
-      if (requisition.width == 0)
-	{
-	  BBS_THREAD_UI_LOCK
-	  {
-	    need_popup_adjustment = TRUE;
-	  }
-	  BBS_THREAD_UI_UNLOCK;
-
-	  use_two_steps_hack = TRUE;
-	}
-      else
-	{
-	  gtk_widget_set_size_request(GTK_WIDGET(popup_context->renderer_context.view),
-				      requisition.width
-				      + THREAD_VIEW_EXTRA_WIDTH,
-				      requisition.height);
-
-	  gtk_window_resize(GTK_WINDOW(popup_context->popup_window),
-			    requisition.width + THREAD_VIEW_EXTRA_WIDTH,
-			    requisition.height);
-	  BBS_THREAD_UI_LOCK
-	  {
-	    need_popup_adjustment = FALSE;
-	  }
-	  BBS_THREAD_UI_UNLOCK;
-	}
-
-      popup_context->y_pos -= (requisition.height + 10);
-      popup_context->y_pos
-	= popup_context->y_pos > 0 ? popup_context->y_pos : 0;
-
-      if (popup_context->renderer_context.application->enable_smart_popup_placement
-	  && screen_width > 0
-	  && (popup_context->x_pos + requisition.width
-	      + THREAD_VIEW_EXTRA_WIDTH) > screen_width)
-	{
-	  popup_context->x_pos =
-	    screen_width - (requisition.width + THREAD_VIEW_EXTRA_WIDTH);
-	  if (popup_context->x_pos < 0)
-	    popup_context->x_pos = 0;
-	}
-
-      gtk_window_move(GTK_WINDOW(popup_context->popup_window),
-		      popup_context->x_pos, popup_context->y_pos);
-      if (use_two_steps_hack)
-	popup_context->y_pos += requisition.height;
-
       gtk_widget_show_all(popup_context->popup_window);
 #if DEBUG_POPUP
       fprintf(stderr, "show_popup_real(): show_all, view=%p\n",
@@ -3240,11 +3341,18 @@ hide_popup_real(PopupContext *popup_context)
 	{
 	  if (popup_context->parent_context != popup_context)
 	    hide_popup(popup_context->parent_context);
-#if DEBUG_POPUP
-	  else
-	    fprintf(stderr, "hide_popup_real(): parent=%p current=%p\n",
-		    popup_context->parent_context, popup_context);
-#endif
+	}
+    }
+  else if (popup_context->image != NULL)
+    {
+      gtk_container_remove(popup_context->popup_frame,
+			   popup_context->image);
+      popup_context->image = NULL;
+
+      if (popup_context->parent_context != NULL)
+	{
+	  if (popup_context->parent_context != popup_context)
+	    hide_popup(popup_context->parent_context);
 	}
     }
 }
@@ -3264,7 +3372,8 @@ popup_close_delay_timeout(PopupContext *popup_context)
   if (!popup_context->now_pointed && !popup_context->now_sticked
       && !(popup_context->renderer_context.view != NULL
 	   && (child_context = g_object_get_qdata(G_OBJECT(popup_context->renderer_context.view), popup_context_id)) != NULL
-	   && child_context->renderer_context.view != NULL))
+	   && (child_context->renderer_context.view != NULL
+	       || child_context->image != NULL)))
     {
 #if DEBUG_POPUP
       fprintf(stderr, "popup_close_delay_timeout: now_pointed=%d child=%p\n",
@@ -3360,8 +3469,8 @@ refresh_thread(OchushaApplication *application, GtkWidget *widget,
 
     if (buffer == old_buffer)
       old_buffer = NULL;
-
-    setup_for_tab_label_animation(buffer, tab_label);
+    else
+      setup_for_tab_label_animation(buffer, tab_label);
 
     info->response_source_buffer = buffer;
     info->rendering_done = FALSE;
