@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bbs_thread_view.c,v 1.42 2004/01/15 09:31:08 fuyu Exp $
+ * $Id: bbs_thread_view.c,v 1.44 2004/01/20 13:50:33 fuyu Exp $
  */
 
 #include "config.h"
@@ -899,8 +899,6 @@ bbs_thread_view_clear(BBSThreadView *view)
 
   g_return_if_fail(IS_BBS_THREAD_VIEW(view));
 
-  CHECK_THREADS_ENTERED();
-
   gtk_text_buffer_get_start_iter(view->text_buffer, &view->iter);
   gtk_text_buffer_get_end_iter(view->text_buffer, &end_iter);
 
@@ -986,7 +984,6 @@ bbs_thread_view_append_text(BBSThreadView *view, const gchar *text, int len)
   int start_offset;
 
   g_return_if_fail(IS_BBS_THREAD_VIEW(view) && text != NULL);
-
   CHECK_THREADS_ENTERED();
 
   args.start = &start;
@@ -1103,7 +1100,8 @@ bbs_thread_view_append_text_as_hidden_link(BBSThreadView *view,
 
 
 void
-bbs_thread_view_append_widget(BBSThreadView *view, GtkWidget *widget)
+bbs_thread_view_append_widget(BBSThreadView *view, GtkWidget *widget,
+			      int hborder, int vborder)
 {
   GtkWidget *vbox;
   GtkWidget *hbox;
@@ -1111,16 +1109,15 @@ bbs_thread_view_append_widget(BBSThreadView *view, GtkWidget *widget)
   guint anchor_offset;
 
   g_return_if_fail(IS_BBS_THREAD_VIEW(view) && GTK_IS_WIDGET(widget));
-
   CHECK_THREADS_ENTERED();
 
   vbox = gtk_vbox_new(FALSE, 0);
   hbox = gtk_hbox_new(FALSE, 0);
 
   gtk_widget_show(widget);
-  gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 30);
+  gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, hborder);
   gtk_widget_show(hbox);
-  gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 20);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, vborder);
   gtk_widget_show(vbox);
 
   anchor_offset = gtk_text_iter_get_offset(&view->iter);
@@ -1174,7 +1171,6 @@ bbs_thread_view_insert_widget_at_offset(BBSThreadView *view, GtkWidget *widget,
   AdjustOffsetArgs args;
 
   g_return_if_fail(IS_BBS_THREAD_VIEW(view) && GTK_IS_WIDGET(widget));
-
   CHECK_THREADS_ENTERED();
 
   vbox = gtk_vbox_new(FALSE, 0);
@@ -1240,7 +1236,6 @@ bbs_thread_view_remove_widget(BBSThreadView *view, GtkWidget *widget)
 
   g_return_if_fail(IS_BBS_THREAD_VIEW(view));
   g_return_if_fail(GTK_IS_WIDGET(widget));
-
   CHECK_THREADS_ENTERED();
 
   anchor_offset = (guint)g_hash_table_lookup(view->child_table, widget);
@@ -1364,7 +1359,6 @@ bbs_thread_view_scroll_to_start(BBSThreadView *view)
   gboolean result;
 
   g_return_if_fail(IS_BBS_THREAD_VIEW(view));
-
   CHECK_THREADS_ENTERED();
 
   buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));
@@ -1385,7 +1379,6 @@ bbs_thread_view_scroll_to_end(BBSThreadView *view)
   gboolean result;
 
   g_return_if_fail(IS_BBS_THREAD_VIEW(view));
-
   CHECK_THREADS_ENTERED();
 
   buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));
@@ -1407,7 +1400,6 @@ bbs_thread_view_scroll_to_offset(BBSThreadView *view, int offset)
 
   g_return_val_if_fail(IS_BBS_THREAD_VIEW(view) && view->text_buffer != NULL,
 		       0);
-
   CHECK_THREADS_ENTERED();
 
   gtk_text_view_place_cursor_onscreen(GTK_TEXT_VIEW(view));
@@ -1431,7 +1423,6 @@ bbs_thread_view_scroll_to_mark(BBSThreadView *view, GtkTextMark *mark)
   GtkTextBuffer *buffer;
 
   g_return_if_fail(IS_BBS_THREAD_VIEW(view) && mark != NULL);
-
   CHECK_THREADS_ENTERED();
 
   buffer = gtk_text_mark_get_buffer(mark);
@@ -1452,6 +1443,7 @@ bbs_thread_view_create_mark_at_offset(BBSThreadView *view, int offset)
 
   g_return_val_if_fail(IS_BBS_THREAD_VIEW(view) && view->text_buffer != NULL,
 		       NULL);
+  CHECK_THREADS_ENTERED();
 
   gtk_text_buffer_get_iter_at_offset(view->text_buffer, &iter, offset);
   mark = gtk_text_buffer_create_mark(view->text_buffer, NULL, &iter, TRUE); 
@@ -1465,6 +1457,7 @@ bbs_thread_view_place_cursor_at_offset(BBSThreadView *view, int offset)
   GtkTextIter iter;
 
   g_return_if_fail(IS_BBS_THREAD_VIEW(view) && view->text_buffer != NULL);
+  CHECK_THREADS_ENTERED();
 
   gtk_text_buffer_get_iter_at_offset(view->text_buffer, &iter, offset);
   gtk_text_buffer_place_cursor(view->text_buffer, &iter);
@@ -1476,6 +1469,9 @@ bbs_thread_view_get_visible_offset(BBSThreadView *view)
 {
   GdkRectangle visible_rect;
   GtkTextIter iter;
+
+  CHECK_THREADS_ENTERED();
+
   gtk_text_view_get_visible_rect(GTK_TEXT_VIEW(view), &visible_rect);
   gtk_text_view_get_iter_at_location(GTK_TEXT_VIEW(view), &iter,
 				     visible_rect.x, visible_rect.y);
@@ -1490,7 +1486,6 @@ bbs_thread_view_make_offset_visible(BBSThreadView *view, int offset)
   gboolean result;
 
   g_return_if_fail(IS_BBS_THREAD_VIEW(view) && view->text_buffer != NULL);
-
   CHECK_THREADS_ENTERED();
 
   gtk_text_buffer_get_iter_at_offset(view->text_buffer, &iter, offset);
