@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: boardlist_ui.c,v 1.78 2004/01/15 15:06:21 fuyu Exp $
+ * $Id: boardlist_ui.c,v 1.81 2004/01/17 09:48:50 fuyu Exp $
  */
 
 #include "config.h"
@@ -56,9 +56,12 @@
 # include <onigposix.h>
 #endif
 
+#include <fcntl.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+
 #include <unistd.h>
 
 #include <zlib.h>
@@ -573,6 +576,160 @@ finalize_boardlist(OchushaApplication *application)
 }
 
 
+static char *tree_view_font_name = NULL;
+static GtkWidget *font_selection_dialog = NULL;
+
+
+static gboolean
+write_gtkrc_for_tree_view_font(OchushaApplication *application,
+			       const char *font_name)
+{
+  int fd = ochusha_config_open_file(&application->config,
+				    OCHUSHA_PREFS_GTKRC, NULL,
+				    O_WRONLY | O_TRUNC | O_CREAT);
+  if (fd >= 0)
+    {
+      FILE *file = fdopen(fd, "w");
+      if (file != NULL)
+	{
+	  fprintf(file, "# " OCHUSHA_PREFS_GTKRC " generated by " PACKAGE_STRING ".\n");
+	  fprintf(file, "# Do NOT Edit!\n\n");
+	  fprintf(file, "style \"ochusha-tree-view-style\"\n{\n");
+	  if (font_name != NULL)
+	    fprintf(file, "  font_name=\"%s\"\n", font_name);
+	  else
+	    fprintf(file, "  font_name=\"\"\n");
+	  fprintf(file, "}\n\n");
+	  fprintf(file, "class \"ThreadlistView\" style \"ochusha-tree-view-style\"\n");
+	  fprintf(file, "class \"BoardlistView\" style \"ochusha-tree-view-style\"\n");
+	  fclose(file);
+	  return TRUE;
+	}
+      else
+	close(fd);
+    }
+  return FALSE;
+}
+
+
+static void
+reload_gtkrc(OchushaApplication *application)
+{
+  char *resource_file = ochusha_config_find_file(&application->config,
+						 OCHUSHA_PREFS_GTKRC, NULL);
+  if (resource_file != NULL)
+    {
+      gtk_rc_parse(resource_file);
+      gtk_rc_add_default_file(resource_file);
+      G_FREE(resource_file);
+    }
+
+  gtk_widget_reset_rc_styles(GTK_WIDGET(application->top_level));
+}
+
+
+static void
+preview_tree_view_font(OchushaApplication *application, const char *font_name)
+{
+  if (write_gtkrc_for_tree_view_font(application, font_name))
+    reload_gtkrc(application);
+}
+
+
+static void
+set_tree_view_font(OchushaApplication *application, const char *font_name)
+{
+  if (write_gtkrc_for_tree_view_font(application, font_name))
+    reload_gtkrc(application);
+}
+
+
+static void
+reset_tree_view_font(OchushaApplication *application)
+{
+  if (write_gtkrc_for_tree_view_font(application, tree_view_font_name))
+    reload_gtkrc(application);
+}
+
+
+static void
+font_selection_dialog_response_cb(GtkWidget *dialog, int response_id,
+				  OchushaApplication *application)
+{
+  GtkFontSelectionDialog *font_selection;
+  gchar *font_name = NULL;
+
+  g_return_if_fail(GTK_IS_FONT_SELECTION_DIALOG(dialog));
+
+  MAIN_THREAD_EVENT_CALLBACK();
+
+  font_selection = GTK_FONT_SELECTION_DIALOG(dialog);
+
+  switch (response_id)
+    {
+    case GTK_RESPONSE_APPLY:
+      font_name = gtk_font_selection_dialog_get_font_name(font_selection);
+      preview_tree_view_font(application, font_name);
+      g_free(font_name);
+
+      ENTER_MAIN_EVENT_LOOP();
+      return;
+
+    case GTK_RESPONSE_OK:
+      font_name	= gtk_font_selection_dialog_get_font_name(font_selection);
+      set_tree_view_font(application, font_name);
+      g_free(font_name);
+      break;
+
+    case GTK_RESPONSE_CANCEL:
+      reset_tree_view_font(application);
+      break;
+    }
+
+  gtk_widget_destroy(dialog);
+
+  if (tree_view_font_name != NULL)
+    {
+      g_free(tree_view_font_name);
+      tree_view_font_name = NULL;
+    }
+
+  ENTER_MAIN_EVENT_LOOP();
+}
+
+
+void
+select_tree_view_font(OchushaApplication *application)
+{
+  PangoContext *context;
+  PangoFontDescription *font_desc;
+  if (font_selection_dialog != NULL)
+    return;
+
+  font_selection_dialog
+    = gtk_font_selection_dialog_new(_("Boardlist and Threadlist View Font Selection"));
+
+  context = gtk_widget_get_pango_context(application->boardlist_view);
+  font_desc = pango_context_get_font_description(context);
+
+  if (tree_view_font_name != NULL)
+    g_free(tree_view_font_name);
+  tree_view_font_name = pango_font_description_to_string(font_desc);
+
+  if (tree_view_font_name != NULL && *tree_view_font_name != '\0')
+    gtk_font_selection_dialog_set_font_name(GTK_FONT_SELECTION_DIALOG(font_selection_dialog),
+					    tree_view_font_name);
+
+  g_signal_connect(G_OBJECT(font_selection_dialog), "response",
+		   G_CALLBACK(font_selection_dialog_response_cb), application);
+  g_signal_connect(G_OBJECT(font_selection_dialog), "destroy",
+		   G_CALLBACK(gtk_widget_destroyed), &font_selection_dialog);
+
+  gtk_widget_show(GTK_FONT_SELECTION_DIALOG(font_selection_dialog)->apply_button);
+  gtk_widget_show_all(font_selection_dialog);
+}
+
+
 static void
 make_new_thread(OchushaApplication *application)
 {
