@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bulletin_board_ui.c,v 1.125 2004/01/17 09:19:37 fuyu Exp $
+ * $Id: bulletin_board_ui.c,v 1.130 2004/01/20 13:51:14 fuyu Exp $
  */
 
 #include "config.h"
@@ -1190,8 +1190,8 @@ refresh_threadlist_view(OchushaApplication *application,
 
     if (info->threadlist_source_buffer == old_buffer)
       old_buffer = NULL;
-
-    setup_for_tab_label_animation(info->threadlist_source_buffer, tab_label);
+    else
+      setup_for_tab_label_animation(info->threadlist_source_buffer, tab_label);
 
     job = G_NEW0(WorkerJob, 1);
     job_args = G_NEW0(ThreadlistJobArgs, 1);
@@ -1475,14 +1475,21 @@ each_thread_cb(OchushaBBSThread *thread, gpointer user_data)
 	}
       else
 	{
-	  OCHU_THREADS_ENTER();
 	  if (ochusha_bbs_thread_get_number_of_responses_read(thread) > 0)
-	    bookmark_thread(args->application->all_threads, thread,
-			    args->application);
+	    {
+	      if ((info->view_flags & BBS_THREAD_COLLECTED) == 0)
+		{
+		  bookmark_thread(args->application->all_threads, thread,
+				  args->application);
+		  info->view_flags |= BBS_THREAD_COLLECTED;
+		}
+	    }
 	  else
-	    bookmark_remove_thread(args->application->all_threads, thread,
-				   args->application);
-	  OCHU_THREADS_LEAVE();
+	    {
+	      bookmark_remove_thread(args->application->all_threads,
+				     thread, args->application);
+	      info->view_flags &= ~BBS_THREAD_COLLECTED;
+	    }
 	}
     }
 
@@ -1504,8 +1511,6 @@ each_thread_cb(OchushaBBSThread *thread, gpointer user_data)
 	  && ((info->view_flags & BBS_THREAD_HIDDEN) == 0
 	      || !args->application->hide_hidden_threads)))
     {
-      OCHU_THREADS_ENTER();
-
       threadlist_view_append_thread(view, thread, info->num_shown,
 				    (MAX(ochusha_bbs_thread_get_number_of_responses_on_server(thread),
 					 ochusha_bbs_thread_get_number_of_responses_read(thread))
@@ -1516,8 +1521,6 @@ each_thread_cb(OchushaBBSThread *thread, gpointer user_data)
 						     args->application->use_builtin_calculate_weight));
       update_threadlist_entry_style(args->application, view, thread);
 
-      OCHU_THREADS_LEAVE();
-
       args->last_thread = thread;
       if (update_condition)
 	info->view_ignored++;
@@ -1528,6 +1531,37 @@ each_thread_cb(OchushaBBSThread *thread, gpointer user_data)
 }
 
 
+typedef struct _ThreadlistRenderingSyncObject
+{
+  pthread_mutex_t lock;
+  pthread_cond_t cond;
+} ThreadlistRenderingSyncObject;
+
+
+static gboolean
+start_rendering(gpointer data)
+{
+  ThreadlistRenderingSyncObject *sync_object
+    = (ThreadlistRenderingSyncObject *)data;
+  if (pthread_mutex_lock(&sync_object->lock) != 0)
+    {
+      fprintf(stderr, "Couldn't lock a mutex.\n");
+      abort();
+    }
+  if (pthread_cond_signal(&sync_object->cond) != 0)
+    {
+      fprintf(stderr, "Couldn't signal a condition.\n");
+      abort();
+    }
+  if (pthread_mutex_unlock(&sync_object->lock) != 0)
+    {
+      fprintf(stderr, "Couldn't unlock a mutex.\n");
+      abort();
+    }
+  return FALSE;
+}
+
+
 static void
 show_threadlist(WorkerThread *employee, gpointer args)
 {
@@ -1549,6 +1583,8 @@ show_threadlist(WorkerThread *employee, gpointer args)
       info->enable_filter,
       job_args->increment_ignored
     };
+  ThreadlistRenderingSyncObject sync_object;
+
 #if DEBUG_GUI_MOST
   {
     gchar *board_name = convert_string(utf8_to_native, board->name, -1);
@@ -1618,8 +1654,59 @@ show_threadlist(WorkerThread *employee, gpointer args)
 
   OCHU_THREADS_LEAVE();
 
+  if (pthread_mutex_init(&sync_object.lock, NULL) != 0)
+    {
+      fprintf(stderr, "Couldn't init a mutex.\n");
+      abort();
+    }
+
+  if (pthread_cond_init(&sync_object.cond, NULL) != 0)
+    {
+      fprintf(stderr, "Couldn't init a condition variable.\n");
+      abort();
+    }
+
+  if (pthread_mutex_lock(&sync_object.lock) != 0)
+    {
+      fprintf(stderr, "Couldn't lock a mutex.\n");
+      abort();
+    }
+
+  /* idleؿϿ */
+  g_idle_add_full(GDK_PRIORITY_REDRAW + 10,
+		  start_rendering, &sync_object, NULL);
+
+  if (pthread_cond_wait(&sync_object.cond, &sync_object.lock) != 0)
+    {
+      fprintf(stderr, "Couldn't wait a condition.\n");
+      abort();
+    }
+
+  if (pthread_mutex_unlock(&sync_object.lock) != 0)
+    {
+      fprintf(stderr, "Couldn't unlock a mutex.\n");
+      abort();
+    }
+
+  if (pthread_mutex_destroy(&sync_object.lock) != 0)
+    {
+      fprintf(stderr, "Couldn't destroy a mutex.\n");
+      abort();
+    }
+
+  if (pthread_cond_destroy(&sync_object.cond) != 0)
+    {
+      fprintf(stderr, "Couldn't destroy a condition.\n");
+      abort();
+    }
+
   if (!ochusha_bulletin_board_refresh_threadlist(board, buffer,
-						 each_thread_cb, &cb_args))
+						 each_thread_cb,
+						 (StartParsingCallback *)OCHU_THREADS_ENTER_FUNC,
+						 (BeforeWaitCallback *)OCHU_THREADS_LEAVE_FUNC,
+						 (AfterWaitCallback *)OCHU_THREADS_ENTER_FUNC,
+						 (EndParsingCallback *)OCHU_THREADS_LEAVE_FUNC,
+						 &cb_args))
     {
 #if DEBUG_GUI_MOST
       gchar *name = convert_string(utf8_to_native, board->name, -1);
@@ -1627,8 +1714,6 @@ show_threadlist(WorkerThread *employee, gpointer args)
       G_FREE(name);
 #endif
       threadlist_view_close(view);
-
-      goto finish_job;
     }
   
  finish_job:
