@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: main.c,v 1.20 2004/01/17 09:48:50 fuyu Exp $
+ * $Id: main.c,v 1.24 2004/01/21 09:56:38 fuyu Exp $
  */
 
 #include "config.h"
@@ -55,8 +55,9 @@
 #include <string.h>
 
 
-#define INITIAL_NUM_THREADS	2
-#define MAXIMUM_NUM_THREADS	32
+#define INITIAL_NUM_THREADS		2
+#define MAXIMUM_NUM_THREADS		32
+#define MAXIMUM_NUM_MODEST_THREADS	4
 
 
 #if DEBUG_GDK_LOCK
@@ -206,7 +207,8 @@ main(int argc, char *argv[])
   main_thread = pthread_self();
 #endif
 
-  initialize_worker(INITIAL_NUM_THREADS, MAXIMUM_NUM_THREADS);
+  initialize_worker(INITIAL_NUM_THREADS, MAXIMUM_NUM_THREADS,
+		    MAXIMUM_NUM_MODEST_THREADS);
 
   resource_file = ochusha_config_find_file(&application->config,
 					   OCHUSHA_GTKRC, NULL);
@@ -244,6 +246,12 @@ main(int argc, char *argv[])
 
   ochusha_init_application(application);
 
+#if 0	/* Ѥθå */
+  g_log_set_always_fatal(G_LOG_LEVEL_WARNING
+			 | G_LOG_LEVEL_CRITICAL
+			 | G_LOG_LEVEL_ERROR);
+#endif
+
   OCHU_THREADS_ENTER();
 
   ENTER_MAIN_EVENT_LOOP();
@@ -348,3 +356,19 @@ main(int argc, char *argv[])
 
   return 0;
 }
+
+
+#if DEBUG_GDK_LOCK
+void
+ochu_threads_enter(void)
+{
+  OCHU_THREADS_ENTER();
+}
+
+
+void
+ochu_threads_leave(void)
+{
+  OCHU_THREADS_LEAVE();
+}
+#endif
