@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_ui.h,v 1.65 2004/01/17 09:45:40 fuyu Exp $
+ * $Id: ochusha_ui.h,v 1.68 2004/01/20 07:51:35 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_UI_H_
@@ -80,7 +80,6 @@
 #define OCHUSHA_DEFAULT_POPUP_CLOSE_DELAY				100
 #define OCHUSHA_DEFAULT_MAXIMUM_NUMBER_OF_POPUP_RESPONSES		5
 
-#define OCHUSHA_DEFAULT_ENABLE_SMART_POPUP_PLACEMENT			TRUE
 #define OCHUSHA_DEFAULT_ANALYZE_INFORMAL_LINKS				TRUE
 
 #define OCHUSHA_DEFAULT_HIDE_HIDDEN_BOARDLIST_ITEMS			FALSE
@@ -90,6 +89,11 @@
 
 #define OCHUSHA_DEFAULT_WEB_BROWSER_TEMPLATE	"mozilla -raise -remote \"openURL(%s,new-window)\""
 
+#define OCHUSHA_DEFAULT_INLINE_IMAGE_WIDTH				160
+#define OCHUSHA_DEFAULT_INLINE_IMAGE_HEIGHT				120
+#define OCHUSHA_DEFAULT_POPUP_IMAGE_WIDTH				480
+#define OCHUSHA_DEFAULT_POPUP_IMAGE_HEIGHT				360
+
 #define THREAD_VIEW_EXTRA_WIDTH			30
 
 #define OCHUSHA_MINIMIZED_ICON			"Ochusha-Minimized"
@@ -231,6 +235,8 @@ struct _OchushaApplication
 
   /* ΤΥǥե */
   ThreadViewMailtoMode show_mailto_mode;
+  gboolean enable_inline_image;
+  gboolean enable_popup_image;
 
 
   gboolean a_bone_by_name;
@@ -252,11 +258,16 @@ struct _OchushaApplication
   gboolean enable_popup_response;
   guint popup_response_delay;
   guint popup_close_delay;
-  gboolean enable_smart_popup_placement;
   int maximum_number_of_popup_responses;
   gboolean analyze_informal_links;
 
 
+  int inline_image_width;
+  int inline_image_height;
+  int popup_image_width;
+  int popup_image_height;
+
+
   /* ֥֥饦Υޥɥ饤ƥץ졼 */
   char *web_browser_template;
 
