@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ugly_gtk2utils.c,v 1.11 2004/01/14 10:03:18 fuyu Exp $
+ * $Id: ugly_gtk2utils.c,v 1.12 2004/01/16 08:01:46 fuyu Exp $
  */
 
 #include "config.h"
@@ -70,11 +70,13 @@ delayed_g_object_unref_real(gpointer data)
     child = NULL;
 #endif
 
-  OCHU_THREADS_ENTER();
+  /* OCHU_THREADS_ENTER(); */
+  gdk_threads_enter();
 
   OCHU_OBJECT_UNREF(data);
 
-  OCHU_THREADS_LEAVE();
+  gdk_threads_leave();
+  /* OCHU_THREADS_LEAVE(); */
 
 #if DEBUG_REF_COUNT
   if (child != NULL)
