@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: virtual_board.c,v 1.15 2004/01/15 02:05:08 fuyu Exp $
+ * $Id: virtual_board.c,v 1.16 2004/01/18 04:50:59 fuyu Exp $
  */
 
 #include "config.h"
@@ -79,6 +79,10 @@ static OchushaAsyncBuffer *virtual_board_get_threadlist_source(
 static gboolean virtual_board_refresh_threadlist(OchushaBulletinBoard *board,
 						 OchushaAsyncBuffer *buffer,
 						 EachThreadCallback *cb,
+						 StartParsingCallback *start_parsing_cb,
+						 BeforeWaitCallback *before_wait_cb,
+						 AfterWaitCallback *after_wait_cb,
+						 EndParsingCallback *end_parsing_cb,
 						 gpointer callback_data);
 
 
@@ -433,6 +437,10 @@ typedef struct _RefreshThreadlistArgs
   OchushaAsyncBuffer *buffer;
   OchushaNetworkBrokerCacheMode mode;
   EachThreadCallback *each_thread_cb;
+  StartParsingCallback *start_parsing_cb;
+  BeforeWaitCallback *before_wait_cb;
+  AfterWaitCallback *after_wait_cb;
+  EndParsingCallback *end_parsing_cb;
   gpointer callback_data;
   GSList *recent_list;
   gboolean terminate_now;
@@ -494,6 +502,10 @@ update_by_board(gpointer list_data, gpointer user_data)
   if (!ochusha_bulletin_board_refresh_threadlist(OCHUSHA_BULLETIN_BOARD(list_data),
 						 buffer,
 						 internal_each_thread_cb,
+						 args->start_parsing_cb,
+						 args->before_wait_cb,
+						 args->after_wait_cb,
+						 args->end_parsing_cb,
 						 user_data))
     {
       /* subject.txtŪʤΤƤߤ줿 */
@@ -506,6 +518,10 @@ static gboolean
 virtual_board_refresh_threadlist(OchushaBulletinBoard *board,
 				 OchushaAsyncBuffer *buffer,
 				 EachThreadCallback *each_thread_cb,
+				 StartParsingCallback *start_parsing_cb,
+				 BeforeWaitCallback *before_wait_cb,
+				 AfterWaitCallback *after_wait_cb,
+				 EndParsingCallback *end_parsing_cb,
 				 gpointer callback_data)
 {
   VirtualBoard *virtual_board;
@@ -518,7 +534,9 @@ virtual_board_refresh_threadlist(OchushaBulletinBoard *board,
       buffer,
       (OchushaNetworkBrokerCacheMode)g_object_get_qdata(G_OBJECT(buffer),
 							cache_mode_id),
-      each_thread_cb, callback_data,
+      each_thread_cb,
+      start_parsing_cb, before_wait_cb, after_wait_cb, end_parsing_cb,
+      callback_data,
       NULL,
       FALSE
     };
