@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha.h,v 1.13 2004/01/15 02:12:14 fuyu Exp $
+ * $Id: ochusha.h,v 1.14 2004/01/18 04:50:26 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_H_
@@ -162,7 +162,10 @@ typedef gboolean BrokenResponseCallback(OchushaBBSThread *thread, int number,
 					gpointer callback_data);
 typedef gboolean EndThreadCallback(OchushaBBSThread *thread, gboolean finished,
 				   gpointer callback_data);
-
+typedef void StartParsingCallback(gpointer callback_data);
+typedef void BeforeWaitCallback(gpointer callback_data);
+typedef void AfterWaitCallback(gpointer callback_data);
+typedef void EndParsingCallback(gpointer callback_data);
 
 struct _OchushaBBSResponse
 {
