@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_async_buffer.c,v 1.24 2004/01/15 02:06:27 fuyu Exp $
+ * $Id: ochusha_async_buffer.c,v 1.26 2004/01/20 13:52:37 fuyu Exp $
  */
 
 #include "config.h"
@@ -224,11 +224,17 @@ static GSList *ochusha_async_buffer_list = NULL;
 
 enum {
   WAKEUP_NOW_SIGNAL,
+
+  ACCESS_STARTED_SIGNAL,
+  ACCESS_PROGRESSED_SIGNAL,
+  ACCESS_FINISHED_SIGNAL,
+  ACCESS_FAILED_SIGNAL,
+
   LAST_SIGNAL
 };
 
 
-static int async_buffer_signals[LAST_SIGNAL] = { 0, };
+static int async_buffer_signals[LAST_SIGNAL] = { 0, 0, 0, 0, 0 };
 
 
 void
@@ -320,8 +326,52 @@ ochusha_async_buffer_class_init(OchushaAsyncBufferClass *klass)
 		 libochusha_marshal_VOID__VOID,
 		 G_TYPE_NONE, 0);
 
+  async_buffer_signals[ACCESS_STARTED_SIGNAL] =
+    g_signal_new("access_started",
+		 G_TYPE_FROM_CLASS(klass),
+		 G_SIGNAL_RUN_LAST,
+		 G_STRUCT_OFFSET(OchushaAsyncBufferClass, access_started),
+		 NULL, NULL,
+		 libochusha_marshal_VOID__VOID,
+		 G_TYPE_NONE, 0);
+  async_buffer_signals[ACCESS_PROGRESSED_SIGNAL] =
+    g_signal_new("access_progressed",
+		 G_TYPE_FROM_CLASS(klass),
+		 G_SIGNAL_RUN_LAST,
+		 G_STRUCT_OFFSET(OchushaAsyncBufferClass, access_progressed),
+		 NULL, NULL,
+		 libochusha_marshal_VOID__INT_INT,
+		 G_TYPE_NONE, 2,
+		 G_TYPE_INT,
+		 G_TYPE_INT);
+  async_buffer_signals[ACCESS_FINISHED_SIGNAL] =
+    g_signal_new("access_finished",
+		 G_TYPE_FROM_CLASS(klass),
+		 G_SIGNAL_RUN_LAST,
+		 G_STRUCT_OFFSET(OchushaAsyncBufferClass, access_finished),
+		 NULL, NULL,
+		 libochusha_marshal_VOID__VOID,
+		 G_TYPE_NONE, 0);
+  async_buffer_signals[ACCESS_FAILED_SIGNAL] =
+    g_signal_new("access_failed",
+		 G_TYPE_FROM_CLASS(klass),
+		 G_SIGNAL_RUN_LAST,
+		 G_STRUCT_OFFSET(OchushaAsyncBufferClass, access_failed),
+		 NULL, NULL,
+		 libochusha_marshal_VOID__INT_STRING,
+		 G_TYPE_NONE, 2,
+		 G_TYPE_INT,
+		 G_TYPE_STRING);
+
   o_class->finalize = ochusha_async_buffer_finalize;
 
+  klass->wakeup_now = NULL;
+
+  klass->access_started = NULL;
+  klass->access_progressed = NULL;
+  klass->access_finished = NULL;
+  klass->access_failed = NULL;
+
   job->canceled = FALSE;
   job->job = force_awake_job;
   job->args = NULL;
@@ -396,7 +446,7 @@ get_sync_object(void)
 
 
 OchushaAsyncBuffer *
-ochusha_async_buffer_new(char *buffer, int length, DestructFunc *destructor)
+ochusha_async_buffer_new(char *buffer, size_t length, DestructFunc *destructor)
 {
   BufferSyncObject *sync_object;
   OchushaAsyncBuffer *buf
@@ -802,7 +852,7 @@ ochusha_async_buffer_terminate(OchushaAsyncBuffer *buffer)
  * ХåեTERMINATED֤ξFALSE֤
  */
 gboolean
-ochusha_async_buffer_update_length(OchushaAsyncBuffer *buffer, int length)
+ochusha_async_buffer_update_length(OchushaAsyncBuffer *buffer, size_t length)
 {
 #if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
   fprintf(stderr, "ochusha_async_buffer_update_length called\n");
@@ -837,7 +887,7 @@ ochusha_async_buffer_fix(OchushaAsyncBuffer *buffer)
  * Out of memory⤷ϥХåեTERMINATED֤ξFALSE֤
  */
 gboolean
-ochusha_async_buffer_resize(OchushaAsyncBuffer *buffer, int length)
+ochusha_async_buffer_resize(OchushaAsyncBuffer *buffer, size_t length)
 {
 #if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
   fprintf(stderr, "ochusha_async_buffer_resize called\n");
@@ -868,7 +918,8 @@ ochusha_async_buffer_resize(OchushaAsyncBuffer *buffer, int length)
  * ХåեTERMINATED֤ξFALSE֤
  */
 gboolean
-ochusha_async_buffer_ensure_free_space(OchushaAsyncBuffer *buffer, int length)
+ochusha_async_buffer_ensure_free_space(OchushaAsyncBuffer *buffer,
+				       size_t length)
 {
   gboolean result = TRUE;
 
@@ -925,7 +976,7 @@ ochusha_async_buffer_ensure_free_space(OchushaAsyncBuffer *buffer, int length)
 
 gboolean
 ochusha_async_buffer_append_data(OchushaAsyncBuffer *buffer, const char *data,
-				 int length)
+				 size_t length)
 {
   gboolean result = TRUE;
 
