@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_async_buffer.h,v 1.11 2004/01/15 02:12:14 fuyu Exp $
+ * $Id: ochusha_async_buffer.h,v 1.13 2004/01/20 13:52:37 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_ASYNCBUF_H_
@@ -70,10 +70,10 @@ struct _OchushaAsyncBuffer
 
   volatile gboolean fixed;
   char volatile *buffer;
-  volatile int length;		/* bufferͭʥǡĹ */
+  volatile size_t length;		/* bufferͭʥǡĹ */
 
   /* ʲprivate */
-  int buffer_length;
+  size_t buffer_length;
   volatile int number_of_active_users;
   volatile int number_of_suspended_users;
   volatile OchushaAsyncBufferStatus state;
@@ -89,13 +89,24 @@ struct _OchushaAsyncBufferClass
 {
   GObjectClass parent_class;
 
+  /* signal handlers */
   void (*wakeup_now)(OchushaAsyncBuffer *buffer);
+
+  /* ΥʥOchushaAsyncBufferΥʥȤΤϡ
+   * ݤʤΤǲ
+   */
+  void (*access_started)(OchushaAsyncBuffer *buffer);
+  void (*access_progressed)(OchushaAsyncBuffer *buffer,
+			    int bytes_read, int bytes_total);
+  void (*access_finished)(OchushaAsyncBuffer *buffer);
+  void (*access_failed)(OchushaAsyncBuffer *buffer, int reason_code,
+			const gchar *reason);
 };
 
 
 GType ochusha_async_buffer_get_type(void);
 
-OchushaAsyncBuffer *ochusha_async_buffer_new(char *buffer, int length,
+OchushaAsyncBuffer *ochusha_async_buffer_new(char *buffer, size_t length,
 					     DestructFunc *destructor);
 OchushaAsyncBuffer *ochusha_async_buffer_new_with_file(int fd);
 OchushaAsyncBuffer *ochusha_async_buffer_new_with_file_mmap(int fd);
@@ -115,13 +126,14 @@ void ochusha_async_buffer_terminate(OchushaAsyncBuffer *buffer);
 
 gboolean ochusha_async_buffer_fix(OchushaAsyncBuffer *buffer);
 gboolean ochusha_async_buffer_update_length(OchushaAsyncBuffer *buffer,
-					    int length);
-gboolean ochusha_async_buffer_resize(OchushaAsyncBuffer *buffer, int length);
+					    size_t length);
+gboolean ochusha_async_buffer_resize(OchushaAsyncBuffer *buffer,
+				     size_t length);
 gboolean ochusha_async_buffer_ensure_free_space(OchushaAsyncBuffer *buffer,
-						int length);
+						size_t length);
 
 gboolean ochusha_async_buffer_append_data(OchushaAsyncBuffer *buffer,
-					  const char *data, int length);
+					  const char *data, size_t length);
 
 gboolean ochusha_async_buffer_read_file(OchushaAsyncBuffer *buffer, int fd);
 
