@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003 The Ochusha Project.
+ * Copyright (c) 2003-2004 The Ochusha Project.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bbs_table.c,v 1.24 2004/01/11 22:29:14 fuyu Exp $
+ * $Id: ochusha_bbs_table.c,v 1.25 2004/01/14 09:59:48 fuyu Exp $
  */
 
 #include "config.h"
@@ -255,7 +255,7 @@ ochusha_bbs_table_prepend_category(OchushaBBSTable *table,
       table->category_list = g_slist_remove(table->category_list, category);
     }
 
-  OCHU_OBJECT_REF(G_OBJECT(category));
+  OCHU_OBJECT_REF(category);
   g_hash_table_insert(table->category_table, category->name, category);
 
   table->category_list = g_slist_prepend(table->category_list, category);
@@ -274,7 +274,7 @@ ochusha_bbs_table_add_category(OchushaBBSTable *table,
       table->category_list = g_slist_remove(table->category_list, category);
     }
 
-  OCHU_OBJECT_REF(G_OBJECT(category));
+  OCHU_OBJECT_REF(category);
   g_hash_table_insert(table->category_table, category->name, category);
 
   table->category_list = g_slist_append(table->category_list, category);
@@ -309,10 +309,10 @@ ochusha_bbs_table_add_board(OchushaBBSTable *table,
   g_return_if_fail(OCHUSHA_IS_BBS_TABLE(table)
 		   && OCHUSHA_IS_BULLETIN_BOARD(board));
 
-  OCHU_OBJECT_REF(G_OBJECT(board));
+  OCHU_OBJECT_REF(board);
   g_hash_table_insert(table->board_table, board->base_url, board);
 
-  OCHU_OBJECT_REF(G_OBJECT(board));
+  OCHU_OBJECT_REF(board);
   g_hash_table_insert(table->board_name_table, board->name, board);
 
   return;
@@ -328,7 +328,7 @@ ochusha_bbs_table_remove_board(OchushaBBSTable *table,
   g_return_if_fail(OCHUSHA_IS_BBS_TABLE(table)
 		   && OCHUSHA_IS_BULLETIN_BOARD(board));
 
-  OCHU_OBJECT_REF(G_OBJECT(board));
+  OCHU_OBJECT_REF(board);
 
   entry = g_hash_table_lookup(table->board_table, board->base_url);
   if (entry == board)
@@ -814,7 +814,7 @@ endElementHandler(void *context, const xmlChar *name)
 		    category = ochusha_board_category_new(category_name);
 		}
 	      else
-		OCHU_OBJECT_REF(G_OBJECT(category));
+		OCHU_OBJECT_REF(category);
 
 #if DEBUG_LIBOCHUSHA_MORE
 	      fprintf(stderr, "emitting read_boardlist_element for category.\n");
@@ -835,7 +835,7 @@ endElementHandler(void *context, const xmlChar *name)
 						   sax_context->board_list);
 	      sax_context->board_list = NULL;
 	      ochusha_bbs_table_add_category(table, category);
-	      OCHU_OBJECT_UNREF(G_OBJECT(category));
+	      OCHU_OBJECT_UNREF(category);
 #if DEBUG_SAX_HANDLER
 	      fprintf(stderr, "adding category: %s\n", category_name);
 #endif
@@ -924,7 +924,7 @@ endElementHandler(void *context, const xmlChar *name)
 				sax_context->board_attributes);
 
 		  ochusha_bbs_table_add_board(table, board);
-		  OCHU_OBJECT_UNREF(G_OBJECT(board));
+		  OCHU_OBJECT_UNREF(board);
 		}
 	    }
 
