@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bbs_thread.c,v 1.26 2004/01/14 09:59:48 fuyu Exp $
+ * $Id: ochusha_bbs_thread.c,v 1.27 2004/01/18 04:50:26 fuyu Exp $
  */
 
 #include "config.h"
@@ -498,6 +498,10 @@ ochusha_bbs_thread_parse_responses(OchushaBBSThread *thread,
 				   EachResponseCallback *each_response_cb,
 				   BrokenResponseCallback *broken_response_cb,
 				   EndThreadCallback *end_thread_cb,
+				   StartParsingCallback *start_parsing_cb,
+				   BeforeWaitCallback *before_wait_cb,
+				   AfterWaitCallback *after_wait_cb,
+				   EndParsingCallback *end_parsing_cb,
 				   gpointer callback_data)
 {
   OchushaBBSThreadClass *klass;
@@ -510,6 +514,8 @@ ochusha_bbs_thread_parse_responses(OchushaBBSThread *thread,
   return (*klass->parse_responses)(thread, buffer, start, number, no_wait,
 				   start_thread_cb, each_response_cb,
 				   broken_response_cb, end_thread_cb,
+				   start_parsing_cb, before_wait_cb,
+				   after_wait_cb, end_parsing_cb,
 				   callback_data);
 }
 
