@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bulletin_board.c,v 1.31 2004/01/15 02:05:08 fuyu Exp $
+ * $Id: ochusha_bulletin_board.c,v 1.32 2004/01/18 04:50:26 fuyu Exp $
  */
 
 #include "config.h"
@@ -665,7 +665,11 @@ ochusha_bulletin_board_get_threadlist_source(OchushaBulletinBoard *board,
 gboolean
 ochusha_bulletin_board_refresh_threadlist(OchushaBulletinBoard *board,
 					  OchushaAsyncBuffer *buffer,
-					  EachThreadCallback *cb,
+					  EachThreadCallback *each_thread_cb,
+					  StartParsingCallback *start_parsing_cb,
+					  BeforeWaitCallback *before_wait_cb,
+					  AfterWaitCallback *after_wait_cb,
+					  EndParsingCallback *end_parsing_cb,
 					  gpointer callback_data)
 {
   OchushaBulletinBoardClass *klass;
@@ -677,7 +681,10 @@ ochusha_bulletin_board_refresh_threadlist(OchushaBulletinBoard *board,
   g_return_val_if_fail(klass->get_threadlist_source != NULL, NULL);
 
   ochusha_bulletin_board_lock_thread_list(board);
-  result = (*klass->refresh_threadlist)(board, buffer, cb, callback_data);
+  result = (*klass->refresh_threadlist)(board, buffer, each_thread_cb,
+					start_parsing_cb, before_wait_cb,
+					after_wait_cb, end_parsing_cb,
+					callback_data);
   ochusha_bulletin_board_unlock_thread_list(board);
 
   return result;
