@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bulletin_board.h,v 1.20 2004/01/15 02:12:14 fuyu Exp $
+ * $Id: ochusha_bulletin_board.h,v 1.21 2004/01/18 04:50:26 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_BULLETIN_BOARD_H_
@@ -140,6 +140,10 @@ struct _OchushaBulletinBoardClass
   gboolean (*refresh_threadlist)(OchushaBulletinBoard *board,
 				 OchushaAsyncBuffer *buffer,
 				 EachThreadCallback *cb,
+				 StartParsingCallback *start_parsing_cb,
+				 BeforeWaitCallback *before_wait_cb,
+				 AfterWaitCallback *after_wait_cb,
+				 EndParsingCallback *end_parsing_cb,
 				 gpointer callback_data);
 
   const char *(*get_response_character_encoding)(OchushaBulletinBoard *board);
@@ -207,6 +211,10 @@ OchushaAsyncBuffer *ochusha_bulletin_board_get_threadlist_source(
 gboolean ochusha_bulletin_board_refresh_threadlist(OchushaBulletinBoard *board,
 						   OchushaAsyncBuffer *buffer,
 						   EachThreadCallback *cb,
+						   StartParsingCallback *start_parsing_cb,
+						   BeforeWaitCallback *before_wait_cb,
+						   AfterWaitCallback *after_wait_cb,
+						   EndParsingCallback *end_parsing_cb,
 						   gpointer callback_data);
 const char *ochusha_bulletin_board_get_response_character_encoding(
 						OchushaBulletinBoard *board);
