@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_config.c,v 1.25 2004/01/15 02:12:14 fuyu Exp $
+ * $Id: ochusha_config.c,v 1.26 2004/01/17 09:28:17 fuyu Exp $
  */
 
 #include "config.h"
@@ -952,7 +952,7 @@ ochusha_write_config_xml(OchushaConfig *config, const char *subdir,
   config_xml = gzdopen(fd, "w");
   if (config_xml == NULL)
     {
-      fprintf(stderr, "Couldn't fdopen for config.xml.\n");
+      fprintf(stderr, "Couldn't gzdopen for config.xml.\n");
       return FALSE;
     }
 
