@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_network_broker.c,v 1.46 2004/01/14 10:11:06 fuyu Exp $
+ * $Id: ochusha_network_broker.c,v 1.49 2004/01/21 08:41:01 fuyu Exp $
  */
 
 #include "config.h"
@@ -100,18 +100,14 @@ ochusha_network_broker_get_type(void)
 
 
 enum {
-  ACCESS_STARTED_SIGNAL,
-  ACCESS_PROGRESSED_SIGNAL,
-  ACCESS_COMPLETED_SIGNAL,
-  ACCESS_TERMINATED_SIGNAL,
-  ACCESS_FAILED_SIGNAL,
   OUTPUT_LOG_SIGNAL,
   LAST_SIGNAL
 };
 
 
+
 static GObjectClass *parent_class = NULL;
-static int broker_signals[LAST_SIGNAL] = { 0, 0, 0, 0, 0, 0 };
+static int broker_signals[LAST_SIGNAL] = { 0 };
 static GQuark broker_id;
 static GQuark broker_buffer_status_id;
 static GQuark broker_job_args_id;
@@ -128,55 +124,6 @@ ochusha_network_broker_class_init(OchushaNetworkBrokerClass *klass)
 
   o_class->finalize = ochusha_network_broker_finalize;
 
-  broker_signals[ACCESS_STARTED_SIGNAL] =
-    g_signal_new("access_started",
-		 G_TYPE_FROM_CLASS(klass),
-		 G_SIGNAL_RUN_LAST,
-		 G_STRUCT_OFFSET(OchushaNetworkBrokerClass, access_started),
-		 NULL, NULL,
-		 libochusha_marshal_BOOLEAN__OBJECT,
-		 G_TYPE_BOOLEAN, 1,
-		 OCHUSHA_TYPE_ASYNC_BUFFER);
-  broker_signals[ACCESS_PROGRESSED_SIGNAL] =
-    g_signal_new("access_progressed",
-		 G_TYPE_FROM_CLASS(klass),
-		 G_SIGNAL_RUN_LAST,
-		 G_STRUCT_OFFSET(OchushaNetworkBrokerClass, access_progressed),
-		 NULL, NULL,
-		 libochusha_marshal_BOOLEAN__OBJECT_INT_INT,
-		 G_TYPE_BOOLEAN, 3,
-		 OCHUSHA_TYPE_ASYNC_BUFFER,
-		 G_TYPE_INT,
-		 G_TYPE_INT);
-  broker_signals[ACCESS_COMPLETED_SIGNAL] =
-    g_signal_new("access_completed",
-		 G_TYPE_FROM_CLASS(klass),
-		 G_SIGNAL_RUN_LAST,
-		 G_STRUCT_OFFSET(OchushaNetworkBrokerClass, access_completed),
-		 NULL, NULL,
-		 libochusha_marshal_BOOLEAN__OBJECT,
-		 G_TYPE_BOOLEAN, 1,
-		 OCHUSHA_TYPE_ASYNC_BUFFER);
-  broker_signals[ACCESS_TERMINATED_SIGNAL] =
-    g_signal_new("access_terminated",
-		 G_TYPE_FROM_CLASS(klass),
-		 G_SIGNAL_RUN_LAST,
-		 G_STRUCT_OFFSET(OchushaNetworkBrokerClass, access_terminated),
-		 NULL, NULL,
-		 libochusha_marshal_BOOLEAN__OBJECT,
-		 G_TYPE_BOOLEAN, 1,
-		 OCHUSHA_TYPE_ASYNC_BUFFER);
-  broker_signals[ACCESS_FAILED_SIGNAL] =
-    g_signal_new("access_failed",
-		 G_TYPE_FROM_CLASS(klass),
-		 G_SIGNAL_RUN_LAST,
-		 G_STRUCT_OFFSET(OchushaNetworkBrokerClass, access_failed),
-		 NULL, NULL,
-		 libochusha_marshal_BOOLEAN__OBJECT_INT_STRING,
-		 G_TYPE_BOOLEAN, 3,
-		 OCHUSHA_TYPE_ASYNC_BUFFER,
-		 G_TYPE_INT,
-		 G_TYPE_STRING);
   broker_signals[OUTPUT_LOG_SIGNAL] =
     g_signal_new("output_log",
 		 G_TYPE_FROM_CLASS(klass),
@@ -198,11 +145,6 @@ ochusha_network_broker_class_init(OchushaNetworkBrokerClass *klass)
   filedesc_id
     = g_quark_from_static_string("OchushaNetworkBroker::FileDescriptor");
 
-  klass->access_started = NULL;
-  klass->access_progressed = NULL;
-  klass->access_completed = NULL;
-  klass->access_terminated = NULL;
-  klass->access_failed = NULL;
   klass->output_log = NULL;
 }
 
@@ -696,7 +638,6 @@ register_polling_function_for_read(WorkerSyncObject *sync_object, int fd)
 static gboolean
 http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
 {
-  gboolean signal_result;
   NetworkBrokerJobArgs *args = g_object_get_qdata(G_OBJECT(buffer),
 					      broker_job_args_id);
   WorkerSyncObject *sync_object = g_object_get_qdata(G_OBJECT(buffer),
@@ -753,13 +694,9 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
 
   if (!setup_common_request_headers(broker, request, FALSE))
     {
-      g_signal_emit(G_OBJECT(broker),
-		    broker_signals[ACCESS_FAILED_SIGNAL],
-		    0,
-		    buffer,
-		    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_INVALID_PROXY,
-		    _("Proxy setting may be wrong."),
-		    &signal_result);
+      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_INVALID_PROXY,
+			_("Proxy setting may be wrong."));
 
       LOCK_WORKER_SYNC_OBJECT(sync_object);
       sync_object->finished = TRUE;
@@ -771,11 +708,7 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
       return FALSE;
     }    
 
-  g_signal_emit(G_OBJECT(broker),
-		broker_signals[ACCESS_STARTED_SIGNAL],
-		0,
-		buffer,
-		&signal_result);
+  g_signal_emit_by_name(G_OBJECT(buffer), "access_started", 0);
 
   snprintf(message, LOG_MESSAGE_SIZE, _("Starting GET request: %s\n"), url);
   ochusha_network_broker_output_log(broker, message);
@@ -905,13 +838,9 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
       if (body_len > 0)
 	{
 	  if (current_status.bytes_total > 0)
-	    g_signal_emit(G_OBJECT(broker),
-			  broker_signals[ACCESS_PROGRESSED_SIGNAL],
-			  0,
-			  buffer,
-			  current_status.bytes_read,
-			  current_status.bytes_total,
-			  &signal_result);
+	    g_signal_emit_by_name(G_OBJECT(buffer), "access_progressed", 0,
+				  current_status.bytes_read,
+				  current_status.bytes_total);
 
 	  if (content_encoding == UNKNOWN_ENCODING)
 	    {
@@ -991,11 +920,6 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
 	    {
 	      if (!ochusha_async_buffer_append_data(buffer, body, body_len))
 		{	/* content_encoding == IDENTITY_ENCODING */
-		  g_signal_emit(G_OBJECT(broker),
-				broker_signals[ACCESS_TERMINATED_SIGNAL],
-				0,
-				buffer,
-				&signal_result);
 		  ghttp_request_destroy(request);
 		  args->request = NULL;
 
@@ -1019,11 +943,6 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
 	{
 	  if (!ochusha_async_buffer_signal(buffer))
 	    {
-	      g_signal_emit(G_OBJECT(broker),
-			    broker_signals[ACCESS_TERMINATED_SIGNAL],
-			    0,
-			    buffer,
-			    &signal_result);
 	      ghttp_request_destroy(request);
 	      args->request = NULL;
 
@@ -1085,11 +1004,7 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
       else
 	status->date = NULL;
 
-      g_signal_emit(G_OBJECT(broker),
-		    broker_signals[ACCESS_COMPLETED_SIGNAL],
-		    0,
-		    buffer,
-		    &signal_result);
+      g_signal_emit_by_name(G_OBJECT(buffer), "access_finished", 0);
 
       ghttp_request_destroy(request);
       args->request = NULL;
@@ -1116,13 +1031,10 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
       snprintf(message, LOG_MESSAGE_SIZE,
 	       _("Access failed(%s): status_code=%d (%s)\n"),
 	       url, status_code, error_message);
-      g_signal_emit(G_OBJECT(broker),
-		    broker_signals[ACCESS_FAILED_SIGNAL],
-		    0,
-		    buffer,
-		    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
-		    message,
-		    &signal_result);
+      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+			    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
+			    message);
+
       ochusha_network_broker_output_log(broker, message);
 #if DEBUG_NETWORK_MOST
       fprintf(stderr, "access failed: status_code=%d %s\n",
@@ -1148,15 +1060,11 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
     }
   else if (status_code != 0)
     {
-      snprintf(message, LOG_MESSAGE_SIZE, _("Access failed(%s): %d (%s)"),
+      snprintf(message, LOG_MESSAGE_SIZE, _("Access failed(%s): %d (%s)\n"),
 	       url, status_code, ghttp_reason_phrase(request));
-      g_signal_emit(G_OBJECT(broker),
-		    broker_signals[ACCESS_FAILED_SIGNAL],
-		    0,
-		    buffer,
-		    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
-		    message,
-		    &signal_result);
+      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+			    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
+			    message);
       ochusha_network_broker_output_log(broker, message);
 #if DEBUG_NETWORK
       {
@@ -1184,13 +1092,6 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
       snprintf(message, LOG_MESSAGE_SIZE,
 	       _("Access failed(%s): unknown reason.\n"),
 	       url);
-      g_signal_emit(G_OBJECT(broker),
-		    broker_signals[ACCESS_FAILED_SIGNAL],
-		    0,
-		    buffer,
-		    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
-		    message,
-		    &signal_result);
       ochusha_network_broker_output_log(broker, message);
 #if DEBUG_NETWORK_MOST
       fprintf(stderr, "%s\n", message);
@@ -1265,7 +1166,8 @@ ochusha_network_broker_employ_worker_thread(OchushaNetworkBroker *broker,
 					    OchushaAsyncBuffer *buffer,
 					    const char *url,
 					    const char *if_modified_since,
-					    JobFunc *job_function)
+					    JobFunc *job_function,
+					    gboolean modestly)
 {
   NetworkBrokerJobArgs *args;
   WorkerJob *job;
@@ -1313,7 +1215,10 @@ ochusha_network_broker_employ_worker_thread(OchushaNetworkBroker *broker,
   OCHU_OBJECT_REF(sync_object);
   OCHU_OBJECT_REF(buffer);
 
-  commit_job(job);
+  if (modestly)
+    commit_modest_job(job);
+  else
+    commit_job(job);
 
   return buffer;
 }
@@ -1322,7 +1227,6 @@ ochusha_network_broker_employ_worker_thread(OchushaNetworkBroker *broker,
 static void
 try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 {
-  gboolean signal_result;
   NetworkBrokerJobArgs *args;
   OchushaNetworkBroker *broker = g_object_get_qdata(G_OBJECT(buffer),
 						    broker_id);
@@ -1347,20 +1251,9 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	url = NULL;
 
       status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
-
-      g_signal_emit(G_OBJECT(broker),
-		    broker_signals[ACCESS_TERMINATED_SIGNAL],
-		    0,
-		    buffer,
-		    &signal_result);
-
-      g_signal_emit(G_OBJECT(broker),
-		    broker_signals[ACCESS_FAILED_SIGNAL],
-		    0,
-		    buffer,
-		    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
-		    _("Access terminated."),
-		    &signal_result);
+      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+		OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
+		_("Access terminated."));
 
       ochusha_async_buffer_fix(buffer);
       g_object_set_qdata(G_OBJECT(buffer), broker_job_args_id, NULL);
@@ -1382,20 +1275,6 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
     {
       status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
 
-      g_signal_emit(G_OBJECT(broker),
-		    broker_signals[ACCESS_TERMINATED_SIGNAL],
-		    0,
-		    buffer,
-		    &signal_result);
-
-      g_signal_emit(G_OBJECT(broker),
-		    broker_signals[ACCESS_FAILED_SIGNAL],
-		    0,
-		    buffer,
-		    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
-		    _("Access terminated."),
-		    &signal_result);
-
       ochusha_async_buffer_fix(buffer);
       g_object_set_qdata(G_OBJECT(buffer), broker_job_args_id, NULL);
       OCHU_OBJECT_UNREF(buffer);
@@ -1416,21 +1295,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
       len = args->length;
       if (!ochusha_async_buffer_update_length(buffer, len))
 	{
-	  g_signal_emit(G_OBJECT(broker),
-			broker_signals[ACCESS_TERMINATED_SIGNAL],
-			0,
-			buffer,
-			&signal_result);
-
 	  status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
-	  g_signal_emit(G_OBJECT(broker),
-			broker_signals[ACCESS_FAILED_SIGNAL],
-			0,
-			buffer,
-			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
-			_("Access terminated."),
-			&signal_result);
-
 	  ochusha_async_buffer_fix(buffer);
 	  g_object_set_qdata(G_OBJECT(buffer), broker_job_args_id, NULL);
 	  OCHU_OBJECT_UNREF(buffer);
@@ -1505,13 +1370,9 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	{
 	  /* ۤäƥåϤɤΤ */
 	  status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_CACHE_IS_OK;
-	  g_signal_emit(G_OBJECT(broker),
-			broker_signals[ACCESS_FAILED_SIGNAL],
-			0,
-			buffer,
+	  g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
 			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_INVALID_PROXY,
-			_("Proxy setting may be wrong."),
-			&signal_result);
+			_("Proxy setting may be wrong."));
 
 	  ghttp_request_destroy(request);
 	  args->request = NULL;
@@ -1519,11 +1380,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	  goto finish_try_update_cache;
 	}
 
-      g_signal_emit(G_OBJECT(broker),
-		    broker_signals[ACCESS_STARTED_SIGNAL],
-		    0,
-		    buffer,
-		    &signal_result);
+      g_signal_emit_by_name(G_OBJECT(buffer), "access_started", 0);
 
       snprintf(message, LOG_MESSAGE_SIZE, _("Updating cache file: %s\n"),
 	       url);
@@ -1537,24 +1394,15 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 
       if (!ochusha_async_buffer_update_length(buffer, offset))
 	{
-	  g_signal_emit(G_OBJECT(broker),
-			broker_signals[ACCESS_TERMINATED_SIGNAL],
-			0,
-			buffer,
-			&signal_result);
 	  status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_CACHE_IS_OK;
-	  g_signal_emit(G_OBJECT(broker),
-			broker_signals[ACCESS_FAILED_SIGNAL],
-			0,
-			buffer,
-			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
-			_("Access terminated."),
-			&signal_result);
 	  ghttp_request_destroy(request);
 	  args->request = NULL;
 
 	  snprintf(message, LOG_MESSAGE_SIZE,
 		   _("Access terminated: %s\n"), url);
+	  g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+		OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
+		message);
 	  ochusha_network_broker_output_log(broker, message);
 
 	  goto finish_try_update_cache;
@@ -1610,11 +1458,10 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 		  status->status_code = status_code;
 		  status->state
 		    = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_CACHE_IS_OK;
-		  g_signal_emit(G_OBJECT(broker),
-				broker_signals[ACCESS_COMPLETED_SIGNAL],
-				0,
-				buffer,
-				&signal_result);
+
+		  g_signal_emit_by_name(G_OBJECT(buffer),
+					"access_finished", 0);
+
 		  ghttp_request_destroy(request);
 		  args->request = NULL;
 
@@ -1699,13 +1546,9 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	  if (body_len > 0)
 	    {
 	      if (current_status.bytes_total > 0)
-		g_signal_emit(G_OBJECT(broker),
-			      broker_signals[ACCESS_PROGRESSED_SIGNAL],
-			      0,
-			      buffer,
-			      current_status.bytes_read,
-			      current_status.bytes_total,
-			      &signal_result);
+		g_signal_emit_by_name(G_OBJECT(buffer), "access_progressed", 0,
+				      current_status.bytes_read,
+				      current_status.bytes_total);
 
 	      if (content_encoding == GZIP_ENCODING
 		  || content_encoding == DEFLATE_ENCODING)
@@ -1732,27 +1575,16 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	      else if (!ochusha_async_buffer_append_data(buffer, body,
 							 body_len))
 		{	/* content_encoding == IDENTITY_ENCODING */
-		  g_signal_emit(G_OBJECT(broker),
-				broker_signals[ACCESS_TERMINATED_SIGNAL],
-				0,
-				buffer,
-				&signal_result);
-
 		  status->state
 		    = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_CACHE_IS_OK;
-		  g_signal_emit(G_OBJECT(broker),
-				broker_signals[ACCESS_FAILED_SIGNAL],
-				0,
-				buffer,
-				OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
-				_("Access terminated."),
-				&signal_result);
-
 		  ghttp_request_destroy(request);
 		  args->request = NULL;
 
 		  snprintf(message, LOG_MESSAGE_SIZE,
 			   _("Access terminated: %s\n"), url);
+		  g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
+			message);
 		  ochusha_network_broker_output_log(broker, message);
 
 		  goto finish_try_update_cache;
@@ -1781,28 +1613,17 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	    {
 	      if (!ochusha_async_buffer_signal(buffer))
 		{
-		  g_signal_emit(G_OBJECT(broker),
-				broker_signals[ACCESS_TERMINATED_SIGNAL],
-				0,
-				buffer,
-				&signal_result);
-
 		  status->state
 		    = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_CACHE_IS_OK;
-		  g_signal_emit(G_OBJECT(broker),
-				broker_signals[ACCESS_FAILED_SIGNAL],
-				0,
-				buffer,
-				OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
-				_("Access terminated."),
-				&signal_result);
-
 		  ghttp_request_destroy(request);
 		  args->request = NULL;
 
 		  snprintf(message, LOG_MESSAGE_SIZE,
 			   _("Access terminated: %s\n"),
 			   url);
+		  g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
+			message);
 		  ochusha_network_broker_output_log(broker, message);
 
 		  goto finish_try_update_cache;
@@ -1846,11 +1667,8 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	status->date = NULL;
 
       status->status_code = status_code;
-      g_signal_emit(G_OBJECT(broker),
-		    broker_signals[ACCESS_COMPLETED_SIGNAL],
-		    0,
-		    buffer,
-		    &signal_result);
+      g_signal_emit_by_name(G_OBJECT(buffer), "access_finished", 0);
+
       ghttp_request_destroy(request);
       args->request = NULL;
       write_buffer_to_cache(broker, url, buffer);
@@ -1874,23 +1692,13 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 
       if (!ochusha_async_buffer_update_length(buffer, 0))
 	{
-	  g_signal_emit(G_OBJECT(broker),
-			broker_signals[ACCESS_TERMINATED_SIGNAL],
-			0,
-			buffer,
-			&signal_result);
 	  status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
 
-	  g_signal_emit(G_OBJECT(broker),
-		broker_signals[ACCESS_FAILED_SIGNAL],
-		0,
-		buffer,
-		OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
-		_("Access terminated."),
-		&signal_result);
-
 	  snprintf(message, LOG_MESSAGE_SIZE,
 		   _("Access terminated: %s\n"), url);
+	  g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+		OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
+		message);
 	  ochusha_network_broker_output_log(broker, message);
 
 	  goto finish_try_update_cache;
@@ -1922,13 +1730,9 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
     {
       snprintf(message, LOG_MESSAGE_SIZE,
 	       _("Access failed(%s): %s\n"), url, error_message);
-      g_signal_emit(G_OBJECT(broker),
-		    broker_signals[ACCESS_FAILED_SIGNAL],
-		    0,
-		    buffer,
-		    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
-		    error_message,
-		    &signal_result);
+      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+			    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
+			    message);
       ochusha_network_broker_output_log(broker, message);
     }
   else
@@ -1936,15 +1740,12 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
       status_code = ghttp_status_code(request);
       if (status_code != 0)
 	{
-	  snprintf(message, LOG_MESSAGE_SIZE, _("Access failed(%s): %d (%s)"),
+	  snprintf(message, LOG_MESSAGE_SIZE,
+		   _("Access failed(%s): %d (%s)\n"),
 		   url, status_code, ghttp_reason_phrase(request));
-	  g_signal_emit(G_OBJECT(broker),
-			broker_signals[ACCESS_FAILED_SIGNAL],
-			0,
-			buffer,
-			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
-			message,
-			&signal_result);
+	  g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+				OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
+				message);
 	  ochusha_network_broker_output_log(broker, message);
 	}
       else
@@ -1952,13 +1753,9 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	  snprintf(message, LOG_MESSAGE_SIZE,
 		   _("Access failed(%s): unknown reason.\n"),
 		   url);
-	  g_signal_emit(G_OBJECT(broker),
-			broker_signals[ACCESS_FAILED_SIGNAL],
-			0,
-			buffer,
-			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
-			message,
-			&signal_result);
+	  g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+				OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
+				message);
 	  ochusha_network_broker_output_log(broker, message);
 	}
 #if DEBUG_NETWORK
@@ -1999,7 +1796,6 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 static void
 refresh_cache_after_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 {
-  gboolean signal_result;
   NetworkBrokerJobArgs *args;
   OchushaNetworkBroker *broker = g_object_get_qdata(G_OBJECT(buffer),
 						    broker_id);
@@ -2014,11 +1810,9 @@ refresh_cache_after_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
   if (!ochusha_async_buffer_active_ref(buffer))
     {
       status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
-      g_signal_emit(G_OBJECT(broker),
-		    broker_signals[ACCESS_TERMINATED_SIGNAL],
-		    0,
-		    buffer,
-		    &signal_result);
+      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+		OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
+		_("Access terminated."));
       goto finish_refresh_cache_after_read;
     }
 
@@ -2026,11 +1820,9 @@ refresh_cache_after_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
   if (args == NULL)
     {
       status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
-      g_signal_emit(G_OBJECT(broker),
-		    broker_signals[ACCESS_TERMINATED_SIGNAL],
-		    0,
-		    buffer,
-		    &signal_result);
+      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+		OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
+		_("Access terminated."));
       goto finish_refresh_cache_after_read;
     }
 
@@ -2055,24 +1847,16 @@ refresh_cache_after_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	    {	/* å夬ä */
 	      if (args->if_modified_since != NULL)
 		status->last_modified = G_STRDUP(args->if_modified_since);
-	      g_signal_emit(G_OBJECT(broker),
-			    broker_signals[ACCESS_COMPLETED_SIGNAL],
-			    0,
-			    buffer,
-			    &signal_result);
+	      g_signal_emit_by_name(G_OBJECT(buffer), "access_finished", 0);
 	    }
 	  else
 	    {	/* ̿˼ԤΤǻʤåȤ */
-	      g_signal_emit(G_OBJECT(broker),
-			    broker_signals[ACCESS_FAILED_SIGNAL],
-			    0,
-			    buffer,
-			    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
-			    _("Couldn't read from network, cache used."),
-			    &signal_result);
 	      snprintf(message, LOG_MESSAGE_SIZE,
 		       _("Use cached file(%s): due to networking error.\n"),
 		       args->url);
+	      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+				OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
+				message);
 	      ochusha_network_broker_output_log(broker, message);
 	    }
 	}
@@ -2081,35 +1865,25 @@ refresh_cache_after_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	  if (status->status_code == 304)
 	    {
 	      /* ٤å夬Ĥʤ */
-	      g_signal_emit(G_OBJECT(broker),
-			broker_signals[ACCESS_FAILED_SIGNAL],
-			0,
-			buffer,
-			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_CACHE_NOT_FOUND,
-			_("Couldn't find cache that should exist."),
-			&signal_result);
 	      snprintf(message, LOG_MESSAGE_SIZE,
 		       _("Couldn't find cache file: %s\n"),
 		       args->url);
+	      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+				OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
+				message);
 	      ochusha_network_broker_output_log(broker, message);
 	    }
 	  else
 	    {
-	      /* ̤˼ */
 	      if (buffer->state != OCHUSHA_ASYNC_BUFFER_TERMINATED)
-		g_signal_emit(G_OBJECT(broker),
-			      broker_signals[ACCESS_FAILED_SIGNAL],
-			      0,
-			      buffer,
-			      OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
-			      _("Couldn't read data via the network."),
-			      &signal_result);
+		g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+				OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
+				_("Couldn't read data via the network."));
 	      else
-		g_signal_emit(G_OBJECT(broker),
-			      broker_signals[ACCESS_TERMINATED_SIGNAL],
-			      0,
-			      buffer,
-			      &signal_result);
+		g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
+			_("Access terminated."));
+
 	    }
 	}
     }
@@ -2126,7 +1900,6 @@ refresh_cache_after_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 static void
 force_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 {
-  gboolean signal_result;
   NetworkBrokerJobArgs *args;
   OchushaNetworkBroker *broker = g_object_get_qdata(G_OBJECT(buffer),
 						    broker_id);
@@ -2140,11 +1913,6 @@ force_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
   if (!ochusha_async_buffer_active_ref(buffer))
     {
       status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
-      g_signal_emit(G_OBJECT(broker),
-		    broker_signals[ACCESS_TERMINATED_SIGNAL],
-		    0,
-		    buffer,
-		    &signal_result);
       goto finish_force_read;
     }
 
@@ -2152,11 +1920,6 @@ force_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
   if (args == NULL)
     {
       status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
-      g_signal_emit(G_OBJECT(broker),
-		    broker_signals[ACCESS_TERMINATED_SIGNAL],
-		    0,
-		    buffer,
-		    &signal_result);
       goto finish_force_read;
     }
 
@@ -2197,7 +1960,6 @@ force_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 static void
 background_read_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 {
-  gboolean signal_result;
   int fd = (int)g_object_get_qdata(G_OBJECT(buffer), filedesc_id);
   OchushaNetworkBroker *broker = g_object_get_qdata(G_OBJECT(buffer),
 						    broker_id);
@@ -2207,11 +1969,6 @@ background_read_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
   if (!ochusha_async_buffer_active_ref(buffer))
     {
       status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
-      g_signal_emit(G_OBJECT(broker),
-		    broker_signals[ACCESS_TERMINATED_SIGNAL],
-		    0,
-		    buffer,
-		    &signal_result);
       goto finish_job;
     }
 
@@ -2219,20 +1976,11 @@ background_read_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 
   if (ochusha_async_buffer_read_file(buffer, fd))
     {
-      g_signal_emit(G_OBJECT(broker),
-		    broker_signals[ACCESS_COMPLETED_SIGNAL],
-		    0,
-		    buffer,
-		    &signal_result);
+      g_signal_emit_by_name(G_OBJECT(buffer), "access_finished", 0);
     }
   else
     {
       status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
-      g_signal_emit(G_OBJECT(broker),
-		    broker_signals[ACCESS_TERMINATED_SIGNAL],
-		    0,
-		    buffer,
-		    &signal_result);
     }
 
   ochusha_async_buffer_active_unref(buffer);
@@ -2276,10 +2024,9 @@ ochusha_network_broker_read_from_url(OchushaNetworkBroker *broker,
 				     OchushaAsyncBuffer *buffer,
 				     const char *url,
 				     const char *if_modified_since,
-				     OchushaNetworkBrokerCacheMode mode)
+				     OchushaNetworkBrokerCacheMode mode,
+				     gboolean modestly)
 {
-  gboolean signal_result;
-
   g_return_val_if_fail(OCHUSHA_IS_NETWORK_BROKER(broker)
 		       && broker->config != NULL && url != NULL, NULL);
 #if DEBUG_NETWORK_MOST
@@ -2288,14 +2035,12 @@ ochusha_network_broker_read_from_url(OchushaNetworkBroker *broker,
 
   if (mode == OCHUSHA_NETWORK_BROKER_CACHE_IGNORE && broker->config->offline)
     {
-      g_signal_emit(G_OBJECT(broker),
-		    broker_signals[ACCESS_FAILED_SIGNAL],
-		    0,
-		    buffer,
-		    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_NOT_CONNECTED,
-		    _("Network not connected on offline mode."),
-		    &signal_result);
-      return NULL;
+#if 0	/* ƤӽФ¦Թǰ褺ȥ */
+      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_NOT_CONNECTED,
+			_("Network not connected on offline mode."));
+#endif
+      return buffer;
     }
 
   if (broker->config->offline
@@ -2331,7 +2076,10 @@ ochusha_network_broker_read_from_url(OchushaNetworkBroker *broker,
 	  OCHU_OBJECT_REF(broker);
 	  OCHU_OBJECT_REF(buffer);
 
-	  commit_job(job);
+	  if (modestly)
+	    commit_modest_job(job);
+	  else
+	    commit_job(job);
 
 	  return buffer;
 	}
@@ -2339,26 +2087,22 @@ ochusha_network_broker_read_from_url(OchushaNetworkBroker *broker,
       if (mode == OCHUSHA_NETWORK_BROKER_CACHE_ONLY)
 	{
 	  /* åʤĥå夷Ȥʤ */
-	  g_signal_emit(G_OBJECT(broker),
-			broker_signals[ACCESS_FAILED_SIGNAL],
-			0,
-			buffer,
+#if 0	/* ƤӽФ¦Թǰ褺ȥ */
+	  g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
 			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_CACHE_NOT_FOUND,
-			_("Couldn't find cache that should exist."),
-			&signal_result);
-	  return NULL;
+			_("Couldn't find cache that should exist."));
+#endif
+	  return buffer;
 	}
 
       if (broker->config->offline)
 	{
-	  g_signal_emit(G_OBJECT(broker),
-			broker_signals[ACCESS_FAILED_SIGNAL],
-			0,
-			buffer,
+#if 0	/* ƤӽФ¦Թǰ褺ȥ */
+	  g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
 			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_NOT_CONNECTED,
-			_("Network not connected on offline mode."),
-			&signal_result);
-	  return NULL;
+			_("Network not connected on offline mode."));
+#endif
+	  return buffer;
 	}
     }
 
@@ -2367,18 +2111,21 @@ ochusha_network_broker_read_from_url(OchushaNetworkBroker *broker,
     case OCHUSHA_NETWORK_BROKER_CACHE_TRY_UPDATE:
       return ochusha_network_broker_employ_worker_thread(broker, buffer,
 						url, if_modified_since,
-						(JobFunc *)try_update_cache);
+						(JobFunc *)try_update_cache,
+						 modestly);
 
     case OCHUSHA_NETWORK_BROKER_CACHE_TRY_REFRESH:
     case OCHUSHA_NETWORK_BROKER_CACHE_AS_IS:
       return ochusha_network_broker_employ_worker_thread(broker, buffer,
 					url, if_modified_since,
-					(JobFunc *)refresh_cache_after_read);
+					(JobFunc *)refresh_cache_after_read,
+					modestly);
 
     case OCHUSHA_NETWORK_BROKER_CACHE_IGNORE:
       return ochusha_network_broker_employ_worker_thread(broker, buffer,
-						    url, NULL,
-						    (JobFunc *)force_read);
+							 url, NULL,
+							 (JobFunc *)force_read,
+							 modestly);
 
     default:
       break;
@@ -2393,7 +2140,6 @@ static gboolean
 http_read_from_url_synchronously(OchushaNetworkBroker *broker,
 				 OchushaAsyncBuffer *buffer)
 {
-  gboolean signal_result;
   NetworkBrokerJobArgs *args = g_object_get_qdata(G_OBJECT(buffer),
 					      broker_job_args_id);
   OchushaNetworkBrokerBufferStatus *status
@@ -2420,24 +2166,16 @@ http_read_from_url_synchronously(OchushaNetworkBroker *broker,
 
   if (!setup_common_request_headers(broker, request, FALSE))
     {
-      g_signal_emit(G_OBJECT(broker),
-		    broker_signals[ACCESS_FAILED_SIGNAL],
-		    0,
-		    buffer,
-		    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_INVALID_PROXY,
-		    _("Proxy setting may be wrong."),
-		    &signal_result);
+      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_INVALID_PROXY,
+			_("Proxy setting may be wrong."));
 
       ghttp_request_destroy(request);
       args->request = NULL;
       return FALSE;
     }    
 
-  g_signal_emit(G_OBJECT(broker),
-		broker_signals[ACCESS_STARTED_SIGNAL],
-		0,
-		buffer,
-		&signal_result);
+  g_signal_emit_by_name(G_OBJECT(buffer), "access_started", 0);
 
   ghttp_set_sync(request, ghttp_sync);
   ghttp_prepare(request);
@@ -2481,13 +2219,9 @@ http_read_from_url_synchronously(OchushaNetworkBroker *broker,
     {
       ghttp_current_status current_status = ghttp_get_status(request);
       if (current_status.bytes_total > 0)
-	g_signal_emit(G_OBJECT(broker),
-		      broker_signals[ACCESS_PROGRESSED_SIGNAL],
-		      0,
-		      buffer,
-		      current_status.bytes_read,
-		      current_status.bytes_total,
-		      &signal_result);
+	g_signal_emit_by_name(G_OBJECT(buffer), "access_progressed", 0,
+			      current_status.bytes_read,
+			      current_status.bytes_total);
 
       content_encoding = get_content_encoding_type(request);
 
@@ -2543,11 +2277,6 @@ http_read_from_url_synchronously(OchushaNetworkBroker *broker,
 	}
       else if (!ochusha_async_buffer_append_data(buffer, body, body_len))
 	{	/* content_encoding == IDENTITY_ENCODING */
-	  g_signal_emit(G_OBJECT(broker),
-			broker_signals[ACCESS_TERMINATED_SIGNAL],
-			0,
-			buffer,
-			&signal_result);
 	  ghttp_request_destroy(request);
 	  args->request = NULL;
 
@@ -2569,11 +2298,7 @@ http_read_from_url_synchronously(OchushaNetworkBroker *broker,
 #if DEBUG_NETWORK_MOST
   fprintf(stderr, "http_read_from_url_synchronously succeeded.\n");
 #endif
-  g_signal_emit(G_OBJECT(broker),
-		broker_signals[ACCESS_COMPLETED_SIGNAL],
-		0,
-		buffer,
-		&signal_result);
+  g_signal_emit_by_name(G_OBJECT(buffer), "access_finished", 0);
 
   return TRUE;
 
@@ -2583,38 +2308,31 @@ http_read_from_url_synchronously(OchushaNetworkBroker *broker,
 #endif
   if (error_message != NULL)
     {
-      g_signal_emit(G_OBJECT(broker),
-		    broker_signals[ACCESS_FAILED_SIGNAL],
-		    0,
-		    buffer,
-		    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
-		    error_message,
-		    &signal_result);
+      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
+			error_message);
+#if 0
       fprintf(stderr, "access failed: status_code=%d %s\n",
 	      status_code, error_message);
+#endif
     }
   else if (status_code != 0)
     {
       char message[LOG_MESSAGE_SIZE];
-      snprintf(message, LOG_MESSAGE_SIZE, "Access failed: %d (%s)",
+      snprintf(message, LOG_MESSAGE_SIZE, "Access failed: %d (%s)\n",
 	       status_code, ghttp_reason_phrase(request));
-      g_signal_emit(G_OBJECT(broker),
-		    broker_signals[ACCESS_FAILED_SIGNAL],
-		    0,
-		    buffer,
-		    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
-		    message,
-		    &signal_result);
+      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
+			message);
+#if 0
       fprintf(stderr, "%s\n", message);
+#endif
     }
   else
-    g_signal_emit(G_OBJECT(broker),
-		  broker_signals[ACCESS_FAILED_SIGNAL],
-		  0,
-		  buffer,
-		  OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
-		  _("Access failed: unknown reason."),
-		  &signal_result);
+    g_signal_emit_by_name(G_OBJECT(buffer), "access_failed", 0,
+			  OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
+			  _("Access failed: unknown reason."));
+
   if (request != NULL)
     ghttp_request_destroy(request);
   args->request = NULL;
@@ -2627,7 +2345,6 @@ http_read_from_url_synchronously(OchushaNetworkBroker *broker,
 static void
 force_read_synchronously(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 {
-  gboolean signal_result;
   NetworkBrokerJobArgs *args;
   OchushaNetworkBroker *broker = g_object_get_qdata(G_OBJECT(buffer),
 						    broker_id);
@@ -2641,11 +2358,6 @@ force_read_synchronously(WorkerThread *employee, OchushaAsyncBuffer *buffer)
   if (!ochusha_async_buffer_active_ref(buffer))
     {
       status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
-      g_signal_emit(G_OBJECT(broker),
-		    broker_signals[ACCESS_TERMINATED_SIGNAL],
-		    0,
-		    buffer,
-		    &signal_result);
       goto finish_force_read;
     }
 
@@ -2653,11 +2365,6 @@ force_read_synchronously(WorkerThread *employee, OchushaAsyncBuffer *buffer)
   if (args == NULL)
     {
       status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
-      g_signal_emit(G_OBJECT(broker),
-		    broker_signals[ACCESS_TERMINATED_SIGNAL],
-		    0,
-		    buffer,
-		    &signal_result);
       goto finish_force_read;
     }
 
@@ -2719,7 +2426,8 @@ ochusha_network_broker_read_from_cgi(OchushaNetworkBroker *broker,
     return NULL;
 
   return ochusha_network_broker_employ_worker_thread(broker, buffer, url, NULL,
-					(JobFunc *)force_read_synchronously);
+					(JobFunc *)force_read_synchronously,
+					FALSE);
 }
 
 
@@ -2780,15 +2488,6 @@ ochusha_network_broker_get_header_names(OchushaNetworkBroker *broker,
 }
 
 
-#if 0
-typedef struct _OchushaNetworkBrokerPostStatus
-{
-  int status_code;
-  char *body;
-  char *set_cookie;
-} OchushaNetworkBrokerPostStatus;
-#endif
-
 gboolean
 ochusha_network_broker_try_post(OchushaNetworkBroker *broker,
 				char *url, const char *server,
@@ -2823,10 +2522,8 @@ ochusha_network_broker_try_post(OchushaNetworkBroker *broker,
   if (cookie != NULL)
     ghttp_set_header(request, "Cookie", cookie);
 
-#if DEBUG_POST
   snprintf(message, LOG_MESSAGE_SIZE, _("Posting a response: %s\n"), url);
   ochusha_network_broker_output_log(broker, message);
-#endif
 
   ghttp_set_body(request, (char *)body, strlen(body));
   ghttp_prepare(request);
