@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_network_broker.h,v 1.9 2004/01/16 23:23:14 fuyu Exp $
+ * $Id: ochusha_network_broker.h,v 1.11 2004/01/21 08:41:01 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_NETWORK_BROKER_H_
@@ -88,19 +88,6 @@ struct _OchushaNetworkBrokerClass
   GObjectClass parent_class;
 
   /* signal handlers */
-  gboolean (*access_started)(OchushaNetworkBroker *broker,
-			     OchushaAsyncBuffer *buffer);
-  gboolean (*access_progressed)(OchushaNetworkBroker *broker,
-				OchushaAsyncBuffer *buffer,
-				int bytes_read, int bytes_total);
-  gboolean (*access_completed)(OchushaNetworkBroker *broker,
-			       OchushaAsyncBuffer *buffer);
-  gboolean (*access_terminated)(OchushaNetworkBroker *broker,
-				OchushaAsyncBuffer *buffer);
-  gboolean (*access_failed)(OchushaNetworkBroker *broker,
-			    OchushaAsyncBuffer *buffer,
-			    OchushaNetworkBrokerFailureReason reason_code,
-			    const gchar *reason);
   void (*output_log)(OchushaNetworkBroker *broker, const gchar *message);
 };
 
@@ -137,7 +124,8 @@ OchushaAsyncBuffer *ochusha_network_broker_read_from_url(
 					OchushaAsyncBuffer *buffer,
 					const char *url,
 					const char *if_modified_since,
-					OchushaNetworkBrokerCacheMode mode);
+					OchushaNetworkBrokerCacheMode mode,
+					gboolean modestly);
 OchushaAsyncBuffer *ochusha_network_broker_read_from_cgi(
 					OchushaNetworkBroker *broker,
 					OchushaAsyncBuffer *buffer,
