@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_thread_2ch.c,v 1.59 2004/01/16 08:00:01 fuyu Exp $
+ * $Id: ochusha_thread_2ch.c,v 1.63 2004/01/21 08:41:01 fuyu Exp $
  */
 
 #include "config.h"
@@ -86,6 +86,10 @@ static gboolean ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
 						   EachResponseCallback *r_cb,
 						   BrokenResponseCallback *bcb,
 						   EndThreadCallback *end_cb,
+						   StartParsingCallback *start_parsing_cb,
+						   BeforeWaitCallback *before_wait_cb,
+						   AfterWaitCallback *after_wait_cb,
+						   EndParsingCallback *end_parsing_cb,
 						   gpointer callback_data);
 
 static time_t ochusha_thread_2ch_get_last_modified_utc(OchushaBBSThread *thread);
@@ -557,7 +561,7 @@ ochusha_thread_2ch_get_responses_source(OchushaBBSThread *thread,
 
   return ochusha_network_broker_read_from_url(broker, buffer,
 				ochusha_thread_2ch_get_dat_url(thread_2ch),
-				thread_2ch->last_modified, mode);
+				thread_2ch->last_modified, mode, FALSE);
 }
 
 
@@ -593,32 +597,34 @@ extract_2ch_response(char *start_pos, char *eol_pos,
   if (tmp_pos != NULL)
     {
       int rest;
-      int clone_len = eol_pos - start_pos;
+      int clone_len = eol_pos - start_pos + 1;
       /* 줿DATǡ */
-      clone_buffer = G_MALLOC(clone_len + 1);
+      clone_buffer = G_MALLOC(clone_len);
       memcpy(clone_buffer, start_pos, clone_len);
+      clone_buffer[clone_len - 1] = '\n';	/* פʤϤǰΤ */
       rest = clone_len;
 
-      cur_pos = clone_buffer + (tmp_pos - start_pos);
+      cur_pos = clone_buffer;
       eol_pos = clone_buffer + clone_len;
       /*
        * DATե1ʬ˴ޤޤƤ롢
        * ٤Ǥʤ'\0''*'֤롣
        */
-      while (*cur_pos == '\0' && *cur_pos != '\n')
+      tmp_pos = cur_pos;
+      while (*tmp_pos != '\n')
 	{
-	  *cur_pos = '*';
-	  cur_pos++;
+	  if (*tmp_pos == '\0')
+	    *tmp_pos = '*';
+	  tmp_pos++;
 	}
 
-      cur_pos = clone_buffer;
 #if DEBUG_DAT
-      clone_buffer[clone_len] = '\0';
+      clone_buffer[clone_len - 1] = '\0';
       fprintf(stderr, "broken line: %s\n", clone_buffer);
 #endif
+      clone_buffer[clone_len - 1] = '\n';
     }
 
-
   tmp_pos = g_strstr_len(cur_pos, eol_pos - cur_pos, "<>");
   if (tmp_pos == NULL)
     goto error_exit;
@@ -635,7 +641,7 @@ extract_2ch_response(char *start_pos, char *eol_pos,
   if (i == 1)
     {
       i = 0;
-      tmp_pos = memchr(start_pos, ',', eol_pos - start_pos);
+      tmp_pos = memchr(cur_pos, ',', eol_pos - cur_pos);
       while (tmp_pos != NULL)
 	{
 	  i++;
@@ -685,11 +691,6 @@ extract_2ch_response(char *start_pos, char *eol_pos,
     return tmp_pos + 2;
 
  error_exit:
-#if DEBUG_DAT_MOST
-  tmp_pos = G_STRNDUP(start_pos, eol_pos - start_pos);
-  fprintf(stderr, "DAT Parsing failed: \"%s\"", tmp_pos);
-  G_FREE(tmp_pos);
-#endif
   if (response->name != NULL)
     {
 #if DEBUG_DAT
@@ -772,6 +773,10 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
 				   EachResponseCallback *each_response_cb,
 				   BrokenResponseCallback *broken_response_cb,
 				   EndThreadCallback *end_thread_cb,
+				   StartParsingCallback *start_parsing_cb,
+				   BeforeWaitCallback *before_wait_cb,
+				   AfterWaitCallback *after_wait_cb,
+				   EndParsingCallback *end_parsing_cb,
 				   gpointer callback_data)
 {
   OchushaThread2ch *thread_2ch;
@@ -828,6 +833,9 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
     int i;
     gboolean buffer_fixed = FALSE;
 
+    if (start_parsing_cb != NULL)
+      (*start_parsing_cb)(callback_data);
+
   retry_analysis:
     rest_of_responses = number;
     offset = 0;
@@ -1004,6 +1012,8 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
 	    if (no_wait)
 	      goto terminated;	/* ξ */
 
+	    if (before_wait_cb != NULL)
+	      (*before_wait_cb)(callback_data);
 	    do
 	      {
 		if (!ochusha_async_buffer_wait(buffer))
@@ -1011,9 +1021,13 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
 #if DEBUG_ASYNC_BUFFER_MOST
 		    fprintf(stderr, "ochusha_thread_2ch_parse_responses(): buffer has been terminated.\n");
 #endif
+		    if (after_wait_cb != NULL)
+		      (*after_wait_cb)(callback_data);
 		    goto terminated;
 		  }
 	      } while (buffer->length == old_len && !buffer->fixed);
+	    if (after_wait_cb != NULL)
+	      (*after_wait_cb)(callback_data);
 	  }
 	else
 	  {
@@ -1024,12 +1038,20 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
 	    g_object_set_qdata(G_OBJECT(buffer), idle_checker_id, buffer);
 	    g_idle_add_full(G_PRIORITY_HIGH_IDLE + 15,
 			    advance_parsing, buffer, NULL);
+	    if (before_wait_cb != NULL)
+	      (*before_wait_cb)(callback_data);
 	    do
 	      {
 		if (!ochusha_async_buffer_wait(buffer))
-		  goto terminated;
+		  {
+		    if (after_wait_cb != NULL)
+		      (*after_wait_cb)(callback_data);
+		    goto terminated;
+		  }
 	      } while (g_object_get_qdata(G_OBJECT(buffer), idle_checker_id)
 		       != NULL);
+	    if (after_wait_cb != NULL)
+	      (*after_wait_cb)(callback_data);
 	    interval = ANALYSIS_INTERVAL;
 	  }
 
@@ -1076,6 +1098,9 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
   if (end_thread_cb != NULL)
     result = (*end_thread_cb)(thread, TRUE, callback_data);
 
+  if (end_parsing_cb != NULL)
+    (*end_parsing_cb)(callback_data);
+
   return TRUE;
 }
 
