@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_thread_jbbs.c,v 1.28 2004/01/16 08:00:01 fuyu Exp $
+ * $Id: ochusha_thread_jbbs.c,v 1.32 2004/01/20 13:52:37 fuyu Exp $
  */
 
 #include "config.h"
@@ -80,6 +80,10 @@ static gboolean ochusha_thread_jbbs_parse_responses(OchushaBBSThread *thread,
 						    EachResponseCallback *r_cb,
 						    BrokenResponseCallback *bcb,
 						    EndThreadCallback *end_cb,
+						    StartParsingCallback *start_parsing_cb,
+						    BeforeWaitCallback *before_wait_cb,
+						    AfterWaitCallback *after_wait_cb,
+						    EndParsingCallback *end_parsing_cb,
 						    gpointer callback_data);
 static const char *ochusha_thread_jbbs_get_base_url(OchushaBBSThread *thread);
 
@@ -302,29 +306,32 @@ extract_jbbs_response(char *start_pos, char *eol_pos,
   if (tmp_pos != NULL)
     {
       int rest;
-      int clone_len = eol_pos - start_pos;
+      int clone_len = eol_pos - start_pos + 1;
       /* 줿DATǡ */
-      clone_buffer = G_MALLOC(clone_len + 1);
+      clone_buffer = G_MALLOC(clone_len);
       memcpy(clone_buffer, start_pos, clone_len);
+      clone_buffer[clone_len - 1] = '\n';	/* פʤϤǰΤ */
       rest = clone_len;
 
-      cur_pos = clone_buffer + (tmp_pos - start_pos);
+      cur_pos = clone_buffer;
       eol_pos = clone_buffer + clone_len;
       /*
        * DATե1ʬ˴ޤޤƤ롢
        * ٤Ǥʤ'\0''*'֤롣
        */
-      while (*cur_pos == '\0' && *cur_pos != '\n')
+      tmp_pos = cur_pos;
+      while (*tmp_pos != '\n')
 	{
-	  *cur_pos = '*';
-	  cur_pos++;
+	  if (*tmp_pos == '\0')
+	    *tmp_pos = '*';
+	  tmp_pos++;
 	}
 
-      rest = clone_len;
 #if DEBUG_DAT
-      clone_buffer[clone_len] = '\0';
+      clone_buffer[clone_len - 1] = '\0';
       fprintf(stderr, "broken line: %s\n", clone_buffer);
 #endif
+      clone_buffer[clone_len - 1] = '\n';
     }
 
 
@@ -481,6 +488,10 @@ ochusha_thread_jbbs_parse_responses(OchushaBBSThread *thread,
 				    EachResponseCallback *each_response_cb,
 				    BrokenResponseCallback *broken_response_cb,
 				    EndThreadCallback *end_thread_cb,
+				    StartParsingCallback *start_parsing_cb,
+				    BeforeWaitCallback *before_wait_cb,
+				    AfterWaitCallback *after_wait_cb,
+				    EndParsingCallback *end_parsing_cb,
 				    gpointer callback_data)
 {
   OchushaThread2ch *thread_2ch;
@@ -537,6 +548,9 @@ ochusha_thread_jbbs_parse_responses(OchushaBBSThread *thread,
     int i;
     gboolean buffer_fixed = FALSE;
 
+    if (start_parsing_cb != NULL)
+      (*start_parsing_cb)(callback_data);
+
   retry_analysis:
     rest_of_responses = number;
     offset = 0;
@@ -688,6 +702,8 @@ ochusha_thread_jbbs_parse_responses(OchushaBBSThread *thread,
 	    if (no_wait)
 	      goto terminated;	/* ξ */
 
+	    if (before_wait_cb != NULL)
+	      (*before_wait_cb)(callback_data);
 	    do
 	      {
 		if (!ochusha_async_buffer_wait(buffer))
@@ -695,9 +711,13 @@ ochusha_thread_jbbs_parse_responses(OchushaBBSThread *thread,
 #if DEBUG_ASYNC_BUFFER_MOST
 		    fprintf(stderr, "ochusha_thread_jbbs_parse_responses(): buffer has been terminated.\n");
 #endif
+		    if (after_wait_cb != NULL)
+		      (*after_wait_cb)(callback_data);
 		    goto terminated;
 		  }
 	      } while (buffer->length == old_len && !buffer->fixed);
+	    if (after_wait_cb != NULL)
+	      (*after_wait_cb)(callback_data);
 	  }
 	else
 	  {
@@ -708,12 +728,20 @@ ochusha_thread_jbbs_parse_responses(OchushaBBSThread *thread,
 	    g_object_set_qdata(G_OBJECT(buffer), idle_checker_id, buffer);
 	    g_idle_add_full(G_PRIORITY_HIGH_IDLE + 15,
 			    advance_parsing, buffer, NULL);
+	    if (before_wait_cb != NULL)
+	      (*before_wait_cb)(callback_data);
 	    do
 	      {
 		if (!ochusha_async_buffer_wait(buffer))
-		  goto terminated;
+		  {
+		    if (after_wait_cb != NULL)
+		      (*after_wait_cb)(callback_data);
+		    goto terminated;
+		  }
 	      } while (g_object_get_qdata(G_OBJECT(buffer), idle_checker_id)
 		       != NULL);
+	    if (after_wait_cb != NULL)
+	      (*after_wait_cb)(callback_data);
 	    interval = ANALYSIS_INTERVAL;
 	  }
 
@@ -754,6 +782,9 @@ ochusha_thread_jbbs_parse_responses(OchushaBBSThread *thread,
   if (end_thread_cb != NULL)
     result = (*end_thread_cb)(thread, TRUE, callback_data);
 
+  if (end_parsing_cb != NULL)
+    (*end_parsing_cb)(callback_data);
+
   return TRUE;
 }
 
@@ -1013,7 +1044,6 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
   OchushaBBSThread *thread = OCHUSHA_BBS_THREAD(thread_2ch);
   int incoming_offset = args->incoming_offset;
   int number_of_cached_responses = args->number_of_cached_responses;
-  gboolean signal_result;
   unsigned int html_offset;
   unsigned int response_number;
   char *title = NULL;
@@ -1140,19 +1170,17 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
 
   if (!ochusha_async_buffer_active_ref(html_buffer))
     {
-      g_signal_emit_by_name(G_OBJECT(args->broker),
-			    "access_terminated",
-			    dat_buffer,
-			    &signal_result);
+      g_signal_emit_by_name(G_OBJECT(dat_buffer), "access_failed", 0,
+		OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
+		_("Access terminated."));
       goto terminated;
     }
 
   if (!ochusha_async_buffer_active_ref(dat_buffer))
     {
-      g_signal_emit_by_name(G_OBJECT(args->broker),
-			    "access_terminated",
-			    dat_buffer,
-			    &signal_result);
+      g_signal_emit_by_name(G_OBJECT(dat_buffer), "access_failed", 0,
+		OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
+		_("Access terminated."));
       ochusha_async_buffer_active_unref(html_buffer);
       goto terminated;
     }
@@ -1460,10 +1488,7 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
 #if DEBUG_HTML_TO_DAT_CONVERTER_MOST
 	  fprintf(stderr, "conversion done.\n");
 #endif
-	  g_signal_emit_by_name(G_OBJECT(args->broker),
-				"access_completed",
-				dat_buffer,
-				&signal_result);
+	  g_signal_emit_by_name(G_OBJECT(dat_buffer), "access_finished", 0);
 
 	  if (previous_line != NULL)
 	    {
@@ -1478,10 +1503,9 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
 #if DEBUG_HTML_TO_DAT_CONVERTER_MOST
 	  fprintf(stderr, "wait terminated.\n");
 #endif
-	  g_signal_emit_by_name(G_OBJECT(args->broker),
-				"access_terminated",
-				dat_buffer,
-				&signal_result);
+	  g_signal_emit_by_name(G_OBJECT(dat_buffer), "access_failed", 0,
+		OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
+		_("Access terminated."));
 	  if (previous_line != NULL)
 	    {
 	      g_free(previous_line);
@@ -1690,13 +1714,9 @@ ochusha_thread_jbbs_fake_responses_source(OchushaThread2ch *thread_2ch,
     {
       if (dat_buffer != NULL)
 	{
-	  gboolean signal_result;
-	  g_signal_emit_by_name(G_OBJECT(broker),
-				"access_failed",
-				dat_buffer,
-				OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
-				_("Access failed: unknown reason"),
-				&signal_result);
+	  g_signal_emit_by_name(G_OBJECT(dat_buffer), "access_failed", 0,
+			  OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
+			  _("Access failed: unknown reason."));
 	  ochusha_async_buffer_fix(dat_buffer);
 	}
       return dat_buffer;
