@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_utils_2ch.c,v 1.35 2004/01/14 09:59:48 fuyu Exp $
+ * $Id: ochusha_utils_2ch.c,v 1.36 2004/01/21 08:41:01 fuyu Exp $
  */
 
 #include "config.h"
@@ -672,7 +672,7 @@ ochusha_utils_2ch_get_bbsmenu(OchushaNetworkBroker *broker)
 {
   return ochusha_network_broker_read_from_url(broker, NULL,
 				broker->config->bbsmenu_url, NULL,
-				OCHUSHA_NETWORK_BROKER_CACHE_IGNORE);
+				OCHUSHA_NETWORK_BROKER_CACHE_IGNORE, FALSE);
 }
 
 
