@@ -23,10 +23,10 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: utils.c,v 1.24 2004/01/17 07:52:25 fuyu Exp $
+ * $Id: utils.c,v 1.26 2004/01/20 07:48:43 fuyu Exp $
  */
 
-#define OCHUSHA_UTILS_IMPLEMENTATION
+#define OCHUSHA_UTILS_IMPLEMENTATION	1
 
 #include "config.h"
 
@@ -46,7 +46,7 @@
 #include <fcntl.h>
 #include <limits.h>
 
-#if TRACE_MEMORY_USAGE
+#if TRACE_MEMORY_USAGE || TRACE_REFCOUNT
 #include <pthread.h>
 #endif
 
@@ -1838,7 +1838,7 @@ delete_refcount_history(GSList *history)
 void
 init_refcount_trace(void)
 {
-  if (pthread_mutex_init(&memory_trace_lock, NULL) != 0)
+  if (pthread_mutex_init(&refcount_trace_lock, NULL) != 0)
     {
       fprintf(stderr, "Couldn't init a mutex.\n");
       abort();
@@ -1877,6 +1877,13 @@ trace_register_g_object(gpointer pointer, const char *file, int line)
 }
 
 
+static void
+output_warning(GObject *object)
+{
+  fprintf(stderr, "%p seems not to be a valid pointer to GObject\n", object);
+}
+
+
 void
 trace_g_object_ref(gpointer pointer, const char *file, int line)
 {
@@ -1885,6 +1892,8 @@ trace_g_object_ref(gpointer pointer, const char *file, int line)
   GObject *object = (GObject *)pointer;
   snprintf(where, 1024, "g_object_ref(%p) at %s, line %d: ref_count=%d",
 	   object, file, line, object->ref_count);
+  if (object->ref_count < 1 || object->ref_count >= 100)
+    output_warning(object);
   REFCOUNT_TRACE_LOCK
   {
     history = g_hash_table_lookup(refcount_trace, object);
@@ -1904,6 +1913,8 @@ trace_g_object_unref(gpointer pointer, const char *file, int line)
   GObject *object = (GObject *)pointer;
   snprintf(where, 1024, "g_object_unref(%p) at %s, line %d: ref_count=%d",
 	   object, file, line, object->ref_count);
+  if (object->ref_count < 1 || object->ref_count >= 100)
+    output_warning(object);
   REFCOUNT_TRACE_LOCK
   {
     history = g_hash_table_lookup(refcount_trace, object);
@@ -1963,19 +1974,10 @@ dump_objects(void)
 }
 
 
-static void
-output_warning(GObject *object)
-{
-  fprintf(stderr, "%p seems not to be a valid pointer to GObject\n", object);
-}
-
-
 void
 ochu_object_ref(gpointer pointer)
 {
   GObject *object = (GObject *)pointer;
-  if (object->ref_count >= 100)
-    output_warning(object);
   OCHU_OBJECT_REF(object);
 }
 
@@ -1984,8 +1986,6 @@ void
 ochu_object_unref(gpointer pointer)
 {
   GObject *object = (GObject *)pointer;
-  if (object->ref_count < 1 || object->ref_count >= 100)
-    output_warning(object);
   OCHU_OBJECT_UNREF(object);
 }
 #endif
