@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003 The Ochusha Project.
+ * Copyright (c) 2003-2004 The Ochusha Project.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: utils.h,v 1.10 2004/01/13 14:52:49 fuyu Exp $
+ * $Id: utils.h,v 1.13 2004/01/14 12:23:23 fuyu Exp $
  */
 
 #ifndef _UTILS_H_
@@ -64,6 +64,11 @@ char *ochusha_utils_url_decode_string(const char *src);
 
 time_t ochusha_utils_get_utc_time(const char *date_string);
 
+const char *mempbrk(const char *string, const char *charset, size_t len);
+const char *reverse_strpbrk(const char *string, const char *charset);
+int parse_int(const gchar *start, size_t len);
+
+
 typedef struct _CGIQueryKeyTuple
 {
   const char *name;
@@ -91,11 +96,11 @@ void dump_pointers(void);
 
 #if TRACE_REFCOUNT
 void init_refcount_trace(void);
-void trace_register_g_object(GObject *object, const char *file, int line);
-void trace_g_object_ref(GObject *object, const char *file, int line);
-void trace_g_object_unref(GObject *object, const char *file, int line);
-void ochu_object_ref(GObject *object);
-void ochu_object_unref(GObject *object);
+void trace_register_g_object(gpointer pointer, const char *file, int line);
+void trace_g_object_ref(gpointer pointer, const char *file, int line);
+void trace_g_object_unref(gpointer pointer, const char *file, int line);
+void ochu_object_ref(gpointer pointer);
+void ochu_object_unref(gpointer pointer);
 void dump_objects(void);
 #endif
 
